require 'spec_helper'

module TCellAgent
  module Hooks
    module V1
      module Frameworks
        module Rails
          module Login
            def self.register_login_event(status, rails_request, user_id, user_valid = nil, password = nil); end
          end
        end
      end
    end
  end

  module Hooks
    module V1
      module Login
        LOGIN_SUCCESS = 'success'.freeze
        LOGIN_FAILURE = 'failure'.freeze

        def self.register_login_event(status, session_id, user_agent, referrer, remote_addr, header_keys, user_id, document_uri, user_valid = nil, password = nil); end
      end
    end
  end

  describe 'manually requiring auth hooks' do
    before(:all) do
      load 'lib/tcell_agent/hooks/login_fraud.rb'
      load 'spec/support/force_logger_mocking.rb'
    end

    describe 'Using generic interface' do
      context 'with a login failure' do
        context 'with login_failed_enabled set to true' do
          it 'should report the login failure' do
            login_fraud = double('login_fraud', :login_failed_enabled => true)
            status = Hooks::V1::Login::LOGIN_FAILURE
            header_keys = %w[HTTP_USER_AGENT HTTP_X_FORWARDED_FOR]
            document_uri = 'http://tcell.tcell.io/login?param_name=param_value'
            referrer = 'http://tcell.tcell.io/?utm_campaign=some-source'

            expect(TCellAgent).to receive(:policy).with(
              TCellAgent::PolicyTypes::LOGINFRAUD
            ).and_return(login_fraud)
            expect(login_fraud).to receive(:report_login_failure).with(
              'user_id',
              nil,
              header_keys,
              nil,
              anything
            )

            Hooks::V1::Login.register_login_event(
              status, 'session_id', 'user_agent', referrer, '1.1.1.1', header_keys, 'user_id', document_uri
            )
          end
        end
      end

      context 'with a login success' do
        context 'with login_success_enabled set to true' do
          it 'should report the login success' do
            login_fraud = double('login_fraud', :login_success_enabled => true)
            status = Hooks::V1::Login::LOGIN_SUCCESS
            header_keys = %w[HTTP_USER_AGENT HTTP_X_FORWARDED_FOR]
            document_uri = 'http://tcell.tcell.io/login?param_name=param_value'
            referrer = 'http://tcell.tcell.io/?utm_campaign=some-source'

            expect(TCellAgent).to receive(:policy).with(TCellAgent::PolicyTypes::LOGINFRAUD).and_return(
              login_fraud
            )
            expect(login_fraud).to receive(:report_login_success).with(
              'user_id',
              header_keys,
              anything
            )

            Hooks::V1::Login.register_login_event(
              status, 'session_id', 'user_agent', referrer, '1.1.1.1', header_keys, 'user_id', document_uri
            )
          end
        end
      end

      context 'with an unknown status' do
        it 'should log the error' do
          login_fraud = double('login_fraud')
          logger = double('logger')

          expect(TCellAgent).to receive(:policy).with(
            TCellAgent::PolicyTypes::LOGINFRAUD
          ).and_return(login_fraud)
          expect(TCellAgent).to_not receive(:send_event)
          expect(TCellAgent::Hooks::LoginFraud).to receive(:get_logger).and_return(logger)
          expect(logger).to receive(:error).with(
            'Unkown login status: mumbo-jumbo'
          )

          status = 'mumbo-jumbo'
          header_keys = %w[HTTP_USER_AGENT HTTP_X_FORWARDED_FOR]
          document_uri = 'http://tcell.tcell.io/login?param_name=param_value'
          referrer = 'http://tcell.tcell.io/?utm_campaign=some-source'

          Hooks::V1::Login.register_login_event(
            status, 'session_id', 'user_agent', referrer, '1.1.1.1', header_keys, 'user_id', document_uri
          )
        end
      end
    end

    describe 'Using rails interface' do
      context 'with a login failure' do
        context 'with login_failed_enabled set to true' do
          it 'should report the login failure' do
            login_fraud = double('login_fraud', :login_failed_enabled => true)
            rails_request = double('rails_request')
            tcell_data = TCellAgent::Instrumentation::TCellData.new
            tcell_data.user_agent = 'user_agent'
            tcell_data.referrer = 'http://tcell.tcell.io/?utm_campaign=some-source'
            tcell_data.remote_address = '1.1.1.1'
            tcell_data.path = 'http://tcell.tcell.io/login?param_name=param_value'
            tcell_data.hmac_session_id = TCellAgent::SensorEvents::Util.hmac('session_id')
            request_env = {
              TCellAgent::Instrumentation::TCELL_ID => tcell_data,
              'HTTP_USER_AGENT' => true,
              'HTTP_X_FORWARDED_FOR' => true
            }

            expect(TCellAgent).to receive(:policy).with(TCellAgent::PolicyTypes::LOGINFRAUD).and_return(
              login_fraud
            )
            expect(rails_request).to receive(:env).and_return(request_env)
            expect(rails_request).to receive(:env).and_return(request_env)
            expect(login_fraud).to receive(:report_login_failure).with(
              'user_id',
              nil,
              request_env,
              nil,
              anything
            )

            status = Hooks::V1::Login::LOGIN_FAILURE

            Hooks::V1::Frameworks::Rails::Login.register_login_event(
              status, rails_request, 'user_id'
            )
          end
        end
      end

      context 'with a login success' do
        context 'with login_success_enabled set to true' do
          it 'should report the login success' do
            login_fraud = double('login_fraud', :login_success_enabled => true)
            rails_request = double('rails_request')
            tcell_data = TCellAgent::Instrumentation::TCellData.new
            tcell_data.user_agent = 'user_agent'
            tcell_data.referrer = 'http://tcell.tcell.io/?utm_campaign=some-source'
            tcell_data.remote_address = '1.1.1.1'
            tcell_data.path = 'http://tcell.tcell.io/login?param_name=param_value'
            tcell_data.hmac_session_id = TCellAgent::SensorEvents::Util.hmac('session_id')
            request_env = {
              TCellAgent::Instrumentation::TCELL_ID => tcell_data,
              'HTTP_USER_AGENT' => true,
              'HTTP_X_FORWARDED_FOR' => true
            }

            expect(TCellAgent).to receive(:policy).with(TCellAgent::PolicyTypes::LOGINFRAUD).and_return(
              login_fraud
            )
            expect(rails_request).to receive(:env).and_return(request_env)
            expect(rails_request).to receive(:env).and_return(request_env)
            expect(login_fraud).to receive(:report_login_success).with(
              'user_id',
              request_env,
              anything
            )

            status = Hooks::V1::Login::LOGIN_SUCCESS

            Hooks::V1::Frameworks::Rails::Login.register_login_event(
              status, rails_request, 'user_id'
            )
          end
        end
      end

      context 'with an unknown status' do
        it 'should log the error' do
          login_fraud = double('login_fraud', :login_failed_enabled => true)
          logger = double('logger')
          rails_request = double('rails_request')
          tcell_data = TCellAgent::Instrumentation::TCellData.new
          tcell_data.user_agent = 'user_agent'
          tcell_data.referrer = 'http://tcell.tcell.io/?utm_campaign=some-source'
          tcell_data.remote_address = '1.1.1.1'
          tcell_data.path = 'http://tcell.tcell.io/login?param_name=param_value'
          tcell_data.hmac_session_id = TCellAgent::SensorEvents::Util.hmac('session_id')
          request_env = {
            TCellAgent::Instrumentation::TCELL_ID => tcell_data,
            'HTTP_USER_AGENT' => true,
            'HTTP_X_FORWARDED_FOR' => true
          }

          expect(TCellAgent).to receive(:policy).with(TCellAgent::PolicyTypes::LOGINFRAUD).and_return(
            login_fraud
          )
          expect(rails_request).to receive(:env).and_return(request_env)
          expect(rails_request).to receive(:env).and_return(request_env)
          expect(TCellAgent).to_not receive(:send_event)
          expect(TCellAgent::Hooks::LoginFraud).to receive(:get_logger).and_return(logger)
          expect(logger).to receive(:error).with(
            'Unkown login status: mumbo-jumbo'
          )

          status = 'mumbo-jumbo'

          Hooks::V1::Frameworks::Rails::Login.register_login_event(
            status, rails_request, 'user_id'
          )
        end
      end
    end
  end
end
