require 'spec_helper'

module TCellAgent
  module Cmdi
    describe '.parse_command' do
      context 'with env' do
        before(:each) do
          @env = { 'TCELL_VAR' => 'enabled' }
        end

        context 'with string command' do
          it 'should parse the command properly' do
            cmd = TCellAgent::Cmdi.parse_command(@env, 'echo')
            expect(cmd).to eq('echo')

            cmd = TCellAgent::Cmdi.parse_command(@env, 'echo', :unsetenv_others => true)
            expect(cmd).to eq('echo')
          end
        end

        context 'with string command and arguments' do
          it 'should parse the command' do
            cmd = TCellAgent::Cmdi.parse_command(@env, 'echo', 'test', :unsetenv_others => true)
            expect(cmd).to eq('echo test')

            cmd = TCellAgent::Cmdi.parse_command(@env, 'echo', 'test', 'one', :unsetenv_others => true)
            expect(cmd).to eq('echo test one')

            cmd = TCellAgent::Cmdi.parse_command(
              @env,
              'magick',
              '-size',
              '320x85',
              'canvas:none',
              '-font',
              'Bookman-DemiItalic',
              '-draw',
              "\"text 25,60 \'Magick\'\"",
              :unsetenv_others => true
            )
            expect(cmd).to eq(
              "magick -size 320x85 canvas:none -font Bookman-DemiItalic -draw \"text 25,60 'Magick'\""
            )
          end
        end

        context 'with array command' do
          it 'should parse the command properly' do
            cmd = TCellAgent::Cmdi.parse_command(@env, %w[echo argv0], :unsetenv_others => true)
            expect(cmd).to eq('echo')
          end
        end

        context 'with array command and arguments' do
          it 'should parse the command properly' do
            cmd = TCellAgent::Cmdi.parse_command(@env, %w[echo argv0], 'test', :unsetenv_others => true)
            expect(cmd).to eq('echo test')

            cmd = TCellAgent::Cmdi.parse_command(@env, %w[echo argv0], 'test', 'one', :unsetenv_others => true)
            expect(cmd).to eq('echo test one')

            cmd = TCellAgent::Cmdi.parse_command(
              @env,
              %w[magick argv0],
              '-size',
              '320x85',
              'canvas:none',
              '-font',
              'Bookman-DemiItalic',
              '-draw',
              "\"text 25,60 \'Magick\'\"",
              :unsetenv_others => true
            )
            expect(cmd).to eq(
              "magick -size 320x85 canvas:none -font Bookman-DemiItalic -draw \"text 25,60 'Magick'\""
            )
          end
        end
      end

      context 'without env' do
        context 'with string command' do
          it 'should parse the command properly' do
            cmd = TCellAgent::Cmdi.parse_command('echo')
            expect(cmd).to eq('echo')

            cmd = TCellAgent::Cmdi.parse_command('echo', :unsetenv_others => true)
            expect(cmd).to eq('echo')
          end
        end

        context 'with string command and arguments' do
          it 'should parse the command' do
            cmd = TCellAgent::Cmdi.parse_command('echo', 'test', :unsetenv_others => true)
            expect(cmd).to eq('echo test')

            cmd = TCellAgent::Cmdi.parse_command('echo', 'test', 'one', :unsetenv_others => true)
            expect(cmd).to eq('echo test one')

            cmd = TCellAgent::Cmdi.parse_command(
              'magick',
              '-size',
              '320x85',
              'canvas:none',
              '-font',
              'Bookman-DemiItalic',
              '-draw',
              "\"text 25,60 \'Magick\'\"",
              :unsetenv_others => true
            )
            expect(cmd).to eq(
              "magick -size 320x85 canvas:none -font Bookman-DemiItalic -draw \"text 25,60 'Magick'\""
            )
          end
        end

        context 'with array command' do
          it 'should parse the command properly' do
            cmd = TCellAgent::Cmdi.parse_command(%w[echo argv0], :unsetenv_others => true)
            expect(cmd).to eq('echo')
          end
        end

        context 'with array command and arguments' do
          it 'should parse the command properly' do
            cmd = TCellAgent::Cmdi.parse_command(%w[echo argv0], 'test', :unsetenv_others => true)
            expect(cmd).to eq('echo test')

            cmd = TCellAgent::Cmdi.parse_command(%w[echo argv0], 'test', 'one', :unsetenv_others => true)
            expect(cmd).to eq('echo test one')

            cmd = TCellAgent::Cmdi.parse_command(
              %w[magick argv0],
              '-size',
              '320x85',
              'canvas:none',
              '-font',
              'Bookman-DemiItalic',
              '-draw',
              "\"text 25,60 \'Magick\'\"",
              :unsetenv_others => true
            )
            expect(cmd).to eq(
              "magick -size 320x85 canvas:none -font Bookman-DemiItalic -draw \"text 25,60 'Magick'\""
            )
          end
        end
      end
    end
    describe '.parse_command_from_open' do
      context 'with empty parameters' do
        it 'should not return a command' do
          cmd = TCellAgent::Cmdi.parse_command_from_open
          expect(cmd).to eq('')
        end
      end
      context 'with empty array' do
        it 'should not return a command' do
          args = []
          cmd = TCellAgent::Cmdi.parse_command_from_open(*args)
          expect(cmd).to eq('')
        end
      end
      context 'with a string command' do
        context 'with an empty string' do
          it 'should return an empty string' do
            cmd = TCellAgent::Cmdi.parse_command_from_open('')
            expect(cmd).to eq('')
          end
        end
        context 'with an empty command' do
          it 'should return an empty string' do
            cmd = TCellAgent::Cmdi.parse_command_from_open('|')
            expect(cmd).to eq('')
          end
        end
        context 'with a non-empty command' do
          it 'should parse the command properly' do
            cmd = TCellAgent::Cmdi.parse_command_from_open('|echo')
            expect(cmd).to eq('echo')

            cmd = TCellAgent::Cmdi.parse_command_from_open('|ls -l /tmp')
            expect(cmd).to eq('ls -l /tmp')
          end
        end
      end
      context 'with a filename argument' do
        it 'should not return a command' do
          cmd = TCellAgent::Cmdi.parse_command_from_open('/tmp')
          expect(cmd).to eq('')
        end
      end
      context 'with a non-string first argument' do
        it 'should return an empty string' do
          cmd = TCellAgent::Cmdi.parse_command_from_open({})
          expect(cmd).to eq('')
        end
      end
    end
  end
end
