# rubocop:disable Style/HashSyntax

require 'spec_helper'
require 'securerandom'

describe 'File' do
  before(:all) do
    native_agent = double('native_agent')
    @local_files_policy = TCellAgent::Policies::LocalFileInclusion.new(
      native_agent, {}
    )
    @system_enablements_policy = TCellAgent::Policies::SystemEnablements.new(
      native_agent, {}
    )
    @filename = get_test_resource_path('lfi_sample_file.txt')
    @file_contents = "This is line one.\nThis is line two.\n"
    @new_file_name = NEW_FILE_NAME
  end

  describe '.new' do
    context 'empty path' do
      it 'should raise an error' do
        expect do
          File.new
        end.to raise_error(ArgumentError)
        expect do
          File.new(nil)
        end.to raise_error(TypeError)
        expect do
          File.new('')
        end.to raise_error(Errno::ENOENT)
      end
    end

    context 'with a file not blocked for read/write' do
      before(:each) do
        expect(TCellAgent).to receive(:policy).with(
          TCellAgent::PolicyTypes::LFI
        ).and_return(@local_files_policy)
        expect(@local_files_policy).to receive(:block_file_access?).and_return(false)
      end

      after(:each) do |example|
      end

      it 'raises an ERRNO::ENOENT error', :file_created => false do
        expect do
          File.new(@new_file_name, 'r')
        end.to raise_error(Errno::ENOENT)
      end

      it 'creates the file when passed mode w' do
        File.new(@new_file_name, 'w')

        expect(File.exist?(@new_file_name)).to be_truthy
        File.delete(@new_file_name) if File.exist?(@new_file_name)
      end

      it 'creates the file with the correct permissions' do
        File.new(@new_file_name, 'w', 0o644)

        expect(File.stat(@new_file_name).mode.to_s(8)[3..5]).to eq('644')
        expect(File.exist?(@new_file_name)).to be_truthy
        File.delete(@new_file_name) if File.exist?(@new_file_name)
      end

      it 'creates the file with the correct permissions' do
        File.new(@new_file_name, 'w', 0o755)

        expect(File.stat(@new_file_name).mode.to_s(8)[3..5]).to eq('755')
        expect(File.exist?(@new_file_name)).to be_truthy
        File.delete(@new_file_name) if File.exist?(@new_file_name)
      end

      it 'reads the file' do
        result = File.new(@filename).read
        expect(result).to eq @file_contents
      end

      it 'reads the file when passed a mode' do
        result = File.new(@filename, 'r').read
        expect(result).to eq @file_contents
      end

      it 'writes to the file' do
        file = File.new('/dev/null', 'w')
        expect(file.write('dummy message')).to eq 13
      end

      it 'appends to the file' do
        file = File.new('/dev/null', 'a')
        expect(file.write('dummy message')).to eq 13
      end

      context 'using mode, permission, binmode opt' do
        after :each do
          expect(@result.binmode?).to be_truthy
          expect(File.stat(@new_file_name).mode.to_s(8)[3..5]).to eq('755')

          File.delete(@new_file_name)
        end

        test_ruby2_ruby3_keywords(File,
                                  'new',
                                  [NEW_FILE_NAME, 'w', 0o755],
                                  { binmode: true },
                                  nil)
      end
    end

    context 'with a file blocked for read/write' do
      before(:each) do
        expect(TCellAgent).to receive(:policy).with(
          TCellAgent::PolicyTypes::LFI
        ).and_return(@local_files_policy)

        expect(@local_files_policy).to receive(:block_file_access?).and_return(true)
      end

      it 'raises an IOError' do
        expect do
          File.new(@new_file_name, 'w')
        end.to raise_error(IOError)
      end

      it 'raises an IOError' do
        expect do
          File.new(@new_file_name, 'w', 644)
        end.to raise_error(IOError)
      end

      it 'raises an IOError' do
        expect do
          File.new(@filename)
        end.to raise_error(IOError)
      end

      it 'raises an IOError' do
        expect do
          File.new(@filename, 'r')
        end.to raise_error(IOError)
      end
      it 'raises an IOError' do
        expect do
          File.new('/dev/null', 'w')
        end.to raise_error(IOError)
      end
    end
  end

  describe '.open' do
    context 'with a file not blocked for read/write' do
      before(:each) do
        expect(TCellAgent).to receive(:policy).with(
          TCellAgent::PolicyTypes::LFI
        ).and_return(@local_files_policy)
        expect(@local_files_policy).to receive(:block_file_access?).and_return(false)
      end

      it 'raises an ERRNO::ENOENT error', :file_created => false do
        expect do
          File.open(@new_file_name, 'r')
        end.to raise_error(Errno::ENOENT)
      end

      it 'creates the file when passed mode w' do
        File.open(@new_file_name, 'w')

        expect(File.exist?(@new_file_name)).to be_truthy
        File.delete(@new_file_name) if File.exist?(@new_file_name)
      end

      it 'creates the file with the correct permissions' do
        File.open(@new_file_name, 'w', 0o644)

        expect(File.stat(@new_file_name).mode.to_s(8)[3..5]).to eq('644')
        expect(File.exist?(@new_file_name)).to be_truthy
        File.delete(@new_file_name) if File.exist?(@new_file_name)
      end

      it 'creates the file with the correct permissions' do
        File.open(@new_file_name, 'w', 0o755)

        expect(File.stat(@new_file_name).mode.to_s(8)[3..5]).to eq('755')
        expect(File.exist?(@new_file_name)).to be_truthy
        File.delete(@new_file_name) if File.exist?(@new_file_name)
      end

      it 'reads the file' do
        result = File.open(@filename).read
        expect(result).to eq @file_contents
      end

      it 'reads the file when passed a mode' do
        result = File.open(@filename, 'r').read
        expect(result).to eq @file_contents
      end

      it 'writes to the file' do
        file = File.open('/dev/null', 'w')
        expect(file.write('dummy message')).to eq 13
      end

      it 'appends to the file' do
        file = File.open('/dev/null', 'a')
        expect(file.write('dummy message')).to eq 13
      end

      context 'using mode, permission, binmode opt' do
        after :each do
          expect(@result.binmode?).to be_truthy
          expect(File.stat(@new_file_name).mode.to_s(8)[3..5]).to eq('755')
        end

        test_ruby2_ruby3_keywords(File,
                                  'open',
                                  [NEW_FILE_NAME, 'w', 0o755],
                                  { binmode: true },
                                  nil)
      end
    end

    context 'with a file blocked for read/write' do
      before(:each) do
        expect(TCellAgent).to receive(:policy).with(
          TCellAgent::PolicyTypes::LFI
        ).and_return(@local_files_policy)

        expect(@local_files_policy).to receive(:block_file_access?).and_return(true)
      end

      it 'raises an IOError' do
        expect do
          File.open(@new_file_name, 'w')
        end.to raise_error(IOError)
      end

      it 'raises an IOError' do
        expect do
          File.open(@new_file_name, 'w', 644)
        end.to raise_error(IOError)
      end

      it 'raises an IOError' do
        expect do
          File.open(@filename)
        end.to raise_error(IOError)
      end

      it 'raises an IOError' do
        expect do
          File.open(@filename, 'r')
        end.to raise_error(IOError)
      end

      it 'raises an IOError' do
        expect do
          File.open('/dev/null', 'w')
        end.to raise_error(IOError)
      end
    end
  end
end

# rubocop:enable Style/HashSyntax
