# rubocop:disable Style/HashSyntax

require 'spec_helper'
require 'securerandom'

FILE_CONTENTS = "This is line one.\nThis is line two.\n".freeze
describe 'IO' do
  before do
    native_agent = double('native_agent')
    @local_files_policy = TCellAgent::Policies::LocalFileInclusion.new(
      native_agent, {}
    )
    @cmdi_policy = TCellAgent::Policies::CommandInjectionPolicy.new(
      native_agent, {}
    )
  end

  before(:all) do
    @filename = get_test_resource_path('lfi_sample_file.txt')
    @file_contents = "This is line one.\nThis is line two.\n"
    @file_length = @file_contents.length

    @new_file_name = NEW_FILE_NAME
    @new_file_contents_offset = "This OFFSETe one.\nThis is line two.\n"
    @new_file_contents_offset_bytes = "\x00\x00\x00\x00\x00OFFSET"
  end

  describe '.binread' do
    context 'empty path' do
      it 'should raise an error' do
        expect do
          IO.binread
        end.to raise_error(ArgumentError)
        expect do
          IO.binread(nil)
        end.to raise_error(TypeError)
        expect do
          IO.binread('')
        end.to raise_error(Errno::ENOENT)
      end
    end
    context 'with a file not blocked for read/write' do
      before do |test|
        unless test.metadata[:skip_before]
          expect(TCellAgent).to receive(:policy).with(
            TCellAgent::PolicyTypes::LFI
          ).and_return(@local_files_policy)
          expect(@local_files_policy).to receive(:block_file_access?).and_return(false)
          expect(TCellAgent::Cmdi).not_to receive(:parse_command_from_open)
        end
      end

      it 'should still be able to execute OS commands', :skip_before do
        result = IO.binread('|echo test')
        expect(result).to eq "test\n"
      end

      it 'should read the file normally' do
        result = IO.binread(@filename)
        expect(result).to eq @file_contents
      end

      it 'should read 20 bytes of the file' do
        result = IO.binread(@filename, 20)
        expect(result).to eq @file_contents[0..19]
      end

      it 'should read 20 bytes starting from the 5th byte' do
        result = IO.binread(@filename, 20, 5)
        expect(result).to eq @file_contents[5..24]
      end
    end
    context 'with a file blocked for read/write' do
      before do |test|
        unless test.metadata[:skip_before]
          expect(TCellAgent).to receive(:policy).with(
            TCellAgent::PolicyTypes::LFI
          ).and_return(@local_files_policy)
          expect(@local_files_policy).to receive(:block_file_access?).and_return(true)
          expect(TCellAgent::Cmdi).not_to receive(:parse_command_from_open)
        end
      end

      it 'should execute shell commands', :skip_before do
        result = IO.binread('|echo test')
        expect(result).to eq "test\n"
      end

      it 'should receive an IOError' do
        expect do
          IO.binread(@filename)
        end.to raise_error(IOError)
      end

      it 'should receive an IOError' do
        expect do
          IO.binread(@filename, 20)
        end.to raise_error(IOError)
      end

      it 'should receive an IOError' do
        expect do
          IO.binread(@filename, 20, 5)
        end.to raise_error(IOError)
      end
    end
  end

  describe '.binwrite' do
    context 'empty path' do
      it 'should raise an error' do
        expect do
          IO.binwrite
        end.to raise_error(ArgumentError)
        expect do
          IO.binwrite(nil)
        end.to raise_error(ArgumentError)
        expect do
          IO.binwrite('')
        end.to raise_error(ArgumentError)
      end
    end

    context 'with a file not blocked for read/write' do
      before do
        expect(TCellAgent).to receive(:policy).with(
          TCellAgent::PolicyTypes::LFI
        ).and_return(@local_files_policy)
        expect(@local_files_policy).to receive(:block_file_access?).and_return(false)
      end

      after(:each) do
        File.delete(@new_file_name)
      end

      context 'with a non-existent file' do
        it 'should create the file' do
          IO.binwrite(@new_file_name, '')
          expect(File.exist?(@new_file_name)).to be_truthy
        end

        it 'should write to the file' do
          expect(IO.binwrite(@new_file_name, @file_contents)).to eq @file_length
        end

        context 'using offset, perm' do
          after :each do
            expect(File.stat(@new_file_name).mode.to_s(8)[3..5]).to eq('444')

            expect(TCellAgent).to receive(:policy).with(
              TCellAgent::PolicyTypes::LFI
            ).and_return(@local_files_policy)
            expect(@local_files_policy).to receive(:block_file_access?).and_return(false)
            expect(File.open(@new_file_name).read).to eq @new_file_contents_offset_bytes
          end

          test_ruby2_ruby3_keywords(IO,
                                    'binwrite',
                                    [NEW_FILE_NAME, 'OFFSET', 5],
                                    { perm: 0o444 },
                                    6)
        end
      end
    end
    context 'with a file blocked for read or write' do
      before do
        expect(TCellAgent).to receive(:policy).with(
          TCellAgent::PolicyTypes::LFI
        ).and_return(@local_files_policy)
        expect(@local_files_policy).to receive(:block_file_access?).and_return(true)
      end

      it 'should not be able to create and write to a file' do
        expect do
          IO.binwrite(@new_file_name, @file_contents)
        end.to raise_error(IOError)
      end
    end
  end

  describe '.foreach' do
    context 'empty path' do
      it 'should raise an error' do
        expect do
          IO.foreach('') { pass }
        end.to raise_error(Errno::ENOENT)
      end
    end
    context 'with a file not blocked for read/write' do
      before do
        expect(TCellAgent).to receive(:policy).with(
          TCellAgent::PolicyTypes::LFI
        ).and_return(@local_files_policy)
        expect(@local_files_policy).to receive(:block_file_access?).and_return(false)
      end

      it 'should be able to read the file' do
        result = ''
        IO.foreach(@filename) { |line| result << line }
        expect(result).to eq @file_contents
      end

      it 'should split each line to a max length of 16' do
        result = ''
        IO.foreach(@filename, 16) { |line| result << line }
        expect(result).to eq @file_contents
      end

      context 'with a filename and sep' do
        # TODO: no documentation on how to use sep
      end
      context 'with a filename and sep and limit' do
        # TODO: no documentation on how to use sep and limit
      end
    end
    context 'with a file blocked for read or write' do
      before do
        expect(TCellAgent).to receive(:policy).with(
          TCellAgent::PolicyTypes::LFI
        ).and_return(@local_files_policy)
        expect(@local_files_policy).to receive(:block_file_access?).and_return(true)
      end

      it 'returns an IOError' do
        expect do
          result = ''
          IO.foreach(@filename) { |line| result << line }
        end.to raise_error(IOError)
      end

      it 'returns an IOError if it is non-existent' do
        expect do
          result = ''
          IO.foreach('/tmp/bad-file-name') { |line| result << line }
        end.to raise_error(IOError)
      end
    end
  end

  describe '.read' do
    context 'empty path' do
      it 'should raise an error' do
        expect do
          IO.read
        end.to raise_error(ArgumentError)
        expect do
          IO.read('')
        end.to raise_error(Errno::ENOENT)
      end
    end
    context 'with a file not blocked for read/write' do
      before do |test|
        unless test.metadata[:skip_before]
          expect(TCellAgent).to receive(:policy).with(
            TCellAgent::PolicyTypes::LFI
          ).and_return(@local_files_policy)

          expect(@local_files_policy).to receive(:block_file_access?).and_return(false)
          expect(TCellAgent::Cmdi).not_to receive(:parse_command_from_open)
        end
      end

      it 'executes OS commands', :skip_before do
        result = IO.read('|echo test')
        expect(result).to eq "test\n"
      end

      it 'returns the file contents' do
        result = IO.read(@filename)
        expect(result).to eq @file_contents
      end

      context 'using length, offset, mode' do
        test_ruby2_ruby3_keywords(IO,
                                  'read',
                                  [get_test_resource_path('lfi_sample_file.txt'), 20, 5],
                                  { mode: 'rb' },
                                  FILE_CONTENTS[5..24])
      end
    end
    context 'with a file blocked for read/write' do
      before do |test|
        unless test.metadata[:skip_before]
          expect(TCellAgent).to receive(:policy).with(
            TCellAgent::PolicyTypes::LFI
          ).and_return(@local_files_policy)

          expect(@local_files_policy).to receive(:block_file_access?).and_return(true)
          expect(TCellAgent::Cmdi).not_to receive(:parse_command_from_open)
        end
      end

      it 'executes OS commands', :skip_before do
        result = IO.read('|echo test')
        expect(result).to eq "test\n"
      end
      it 'receeive an IOError' do
        expect do
          IO.read(@filename)
        end.to raise_error(IOError)
      end
      it 'receeive an IOError' do
        expect do
          IO.read(@filename, 20)
        end.to raise_error(IOError)
      end
      it 'receeive an IOError' do
        expect do
          IO.read(@filename, 20, 5)
        end.to raise_error(IOError)
      end
    end
  end

  describe '.readlines' do
    context 'empty path' do
      it 'should raise an error' do
        expect do
          IO.readlines
        end.to raise_error(ArgumentError)
        expect do
          IO.readlines('')
        end.to raise_error(Errno::ENOENT)
      end
    end
    context 'with a file not blocked for read/write' do
      before do |test|
        unless test.metadata[:skip_before]
          expect(TCellAgent).to receive(:policy).with(
            TCellAgent::PolicyTypes::LFI
          ).and_return(@local_files_policy)

          expect(@local_files_policy).to receive(:block_file_access?).and_return(false)
          expect(TCellAgent::Cmdi).not_to receive(:parse_command_from_open)
        end
      end

      it 'reads the file normally' do
        result = IO.readlines(@filename)
        expect(result[0] + result[1]).to eq @file_contents
      end

      context 'using limit, separator' do
        it 'splits the file and splits it into 5 char chunks' do
          exp_result = ['This ', 'is li', 'ne on', "e.\n", 'This ', 'is li', 'ne tw', "o.\n"]
          result = IO.readlines(@filename, 5)
          expect(result).to eq exp_result
        end

        it 'splits the file based on the sep and limit' do
          exp_result = ['This ', 'is ', 'line ', "one.\n", 'This ', 'is ', 'line ', "two.\n"]
          result = IO.readlines(@filename, ' ', 5)
          expect(result).to eq exp_result
        end

        test_ruby2_ruby3_keywords(IO,
                                  'readlines',
                                  [get_test_resource_path('lfi_sample_file.txt'), ' ', 5],
                                  { chomp: true },
                                  ['This', 'is', 'line', "one.\n", 'This', 'is', 'line', "two.\n"],
                                  '2.4.0')
      end
    end
    context 'with a file blocked for read/write' do
      before do |test|
        unless test.metadata[:skip_before]
          expect(TCellAgent).to receive(:policy).with(
            TCellAgent::PolicyTypes::LFI
          ).and_return(@local_files_policy)

          expect(@local_files_policy).to receive(:block_file_access?).and_return(true)
          expect(TCellAgent::Cmdi).not_to receive(:parse_command_from_open)
        end
      end

      it 'should still be able to execute OS commands', :skip_before do
        result = IO.readlines('|echo test')[0]
        expect(result).to eq "test\n"
      end

      it 'should not be able to read the file' do
        expect do
          IO.readlines(@filename)
        end.to raise_error(IOError)
      end
    end
  end

  describe '.sysopen' do
    context 'empty path' do
      it 'should raise an error' do
        expect do
          IO.sysopen
        end.to raise_error(ArgumentError)
        expect do
          IO.sysopen(nil)
        end.to raise_error(TypeError)
        expect do
          IO.sysopen('')
        end.to raise_error(Errno::ENOENT)
      end
    end

    context 'with a file not blocked for read/write' do
      before do
        expect(TCellAgent).to receive(:policy).with(
          TCellAgent::PolicyTypes::LFI
        ).and_return(@local_files_policy)
        expect(@local_files_policy).to receive(:block_file_access?).and_return(false)
      end

      it 'returns an integer' do
        fd = IO.sysopen(@filename)
        expect(fd).to be_a(Integer)
      end
      it 'opens the file for read' do
        fd = IO.sysopen(@filename)
        expect(IO.new(fd).read).to eq @file_contents
      end
      it 'uses the mode when set' do
        fd = IO.sysopen(@new_file_name, 'w')
        expect(fd).to be_a(Integer)

        file = IO.new(fd, 'w')
        file.puts @file_contents
        file.rewind
      end
      it 'should open the file for write' do
        fd = IO.sysopen(@new_file_name, 'w')
        file = IO.new(fd, 'w')
        file.puts @file_contents
        file.rewind
      end
    end
    context 'with a file blocked for read/write' do
      before do
        expect(TCellAgent).to receive(:policy).with(
          TCellAgent::PolicyTypes::LFI
        ).and_return(@local_files_policy)
        expect(@local_files_policy).to receive(:block_file_access?).and_return(true)
      end

      it 'should not be able to open the file for read' do
        expect do
          IO.sysopen(@filename)
        end.to raise_error(IOError)
      end

      it 'should not be able to open the file for read' do
        expect do
          IO.sysopen(@filename, 'r')
        end.to raise_error(IOError)
      end
    end
  end

  describe '.write' do
    context 'empty path' do
      it 'should raise an error' do
        expect do
          IO.write
        end.to raise_error(ArgumentError)
        expect do
          IO.write(nil)
        end.to raise_error(ArgumentError)
        expect do
          IO.write('')
        end.to raise_error(ArgumentError)
      end
    end
    context 'with a file not blocked for read/write' do
      before do
        expect(TCellAgent).to receive(:policy).with(
          TCellAgent::PolicyTypes::LFI
        ).and_return(@local_files_policy)
        expect(@local_files_policy).to receive(:block_file_access?).and_return(false)
      end

      after :each do
        File.delete(@new_file_name)
      end
      it 'creates the file' do
        IO.write(@new_file_name, @file_contents)
        expect(File.exist?(@new_file_name)).to be_truthy
      end
      it 'writes to the file' do
        expect(TCellAgent).to receive(:policy).with(
          TCellAgent::PolicyTypes::LFI
        ).and_return(@local_files_policy)
        expect(@local_files_policy).to receive(:block_file_access?).and_return(false)

        IO.write(@new_file_name, @file_contents)
        expect(File.open(@new_file_name).read).to eq @file_contents
      end

      it 'writes over the file' do
        expect(TCellAgent).to receive(:policy).with(
          TCellAgent::PolicyTypes::LFI
        ).and_return(@local_files_policy)
        expect(@local_files_policy).to receive(:block_file_access?).and_return(false)

        IO.write(@new_file_name, @file_contents)
        expect(File.open(@new_file_name).read).to eq @file_contents
      end

      context 'using offset, perm' do
        after :each do
          expect(File.stat(@new_file_name).mode.to_s(8)[3..5]).to eq('444')

          expect(TCellAgent).to receive(:policy).with(
            TCellAgent::PolicyTypes::LFI
          ).and_return(@local_files_policy)
          expect(@local_files_policy).to receive(:block_file_access?).and_return(false)
          expect(File.open(@new_file_name).read).to eq @new_file_contents_offset_bytes
        end

        test_ruby2_ruby3_keywords(IO,
                                  'write',
                                  [NEW_FILE_NAME, 'OFFSET', 5],
                                  { perm: 0o444 },
                                  6)
      end
    end
    context 'with a file blocked for read/write' do
      before do
        expect(TCellAgent).to receive(:policy).with(
          TCellAgent::PolicyTypes::LFI
        ).and_return(@local_files_policy)
        expect(@local_files_policy).to receive(:block_file_access?).and_return(true)
      end

      it 'raises an IOError' do
        expect do
          IO.write(@new_file_name, '')
        end.to raise_error(IOError)
      end
      it 'should not be able to create and write to the file' do
        expect do
          IO.write(@filename, @file_contents)
        end.to raise_error(IOError)
      end
    end
  end
end

# rubocop:enable Style/HashSyntax
