# rubocop:disable Security/Open
# rubocop:disable Lint/UselessAssignment
require 'spec_helper'
require 'securerandom'
require 'pathname'

describe 'Kernel' do
  before do
    native_agent = double('native_agent')
    @local_files_policy = TCellAgent::Policies::LocalFileInclusion.new(
      native_agent, {}
    )
    @filename = get_test_resource_path('lfi_sample_file.txt')
    @file_contents = "This is line one.\nThis is line two.\n"
  end

  before(:all) do
    @new_file_name = NEW_FILE_NAME
    @new_pathname = Pathname.new(@new_file_name)
  end
  describe '#open and ::open' do
    context 'empty path' do
      it 'raises an error' do
        expect do
          Kernel.open
        end.to raise_error(ArgumentError)
        expect do
          Kernel.open(nil)
        end.to raise_error(TypeError)
        expect do
          Kernel.open('')
        end.to raise_error(Errno::ENOENT)
        expect do
          open
        end.to raise_error(ArgumentError)
        expect do
          open(nil)
        end.to raise_error(TypeError)
        expect do
          open('')
        end.to raise_error(Errno::ENOENT)
      end
    end
    context 'with filename not blocked for read/write' do
      before do |test|
        unless test.metadata[:skip_before]
          expect(TCellAgent).to receive(:policy).with(
            TCellAgent::PolicyTypes::LFI
          ).and_return(@local_files_policy, @local_files_policy)
          expect(@local_files_policy).to receive(:block_file_access?).and_return(false, false)
          expect(TCellAgent::Cmdi).not_to receive(:parse_command_from_open)
        end
      end

      it 'executes OS commands', :skip_before do
        result = Kernel.open('|echo test').read
        expect(result).to eq "test\n"

        result = open('|echo test').read
        expect(result).to eq "test\n"
      end

      it 'creates the file when passed a pathname' do
        Kernel.open(@new_pathname, 'w')
        expect(File.exist?(@new_pathname)).to be_truthy
        File.delete(@new_pathname)

        open(@new_pathname, 'w')
        expect(File.exist?(@new_pathname)).to be_truthy
        File.delete(@new_pathname)
      end

      it 'creates the file when passed a string' do
        Kernel.open(@new_file_name, 'w')
        expect(File.exist?(@new_file_name)).to be_truthy
        File.delete(@new_file_name)

        open(@new_file_name, 'w')
        expect(File.exist?(@new_file_name)).to be_truthy
        File.delete(@new_file_name)
      end

      it 'creates the file with the permission 644' do
        Kernel.open(@new_file_name, 'w', 0o644)
        expect(File.stat(@new_file_name).mode.to_s(8)[3..5]).to eq('644')
        File.delete(@new_file_name)

        open(@new_file_name, 'w', 0o644)
        expect(File.stat(@new_file_name).mode.to_s(8)[3..5]).to eq('644')
        File.delete(@new_file_name)
      end

      it 'creates the file with the permission 755' do
        Kernel.open(@new_file_name, 'w', 0o777)
        expect(File.stat(@new_file_name).mode.to_s(8)[3..5]).to eq('755')
        File.delete(@new_file_name)

        open(@new_file_name, 'w', 0o777)
        expect(File.stat(@new_file_name).mode.to_s(8)[3..5]).to eq('755')
        File.delete(@new_file_name)
      end

      context 'using mode, perm, binmode', :skip_before do
        before(:each) do
          expect(TCellAgent).to receive(:policy).with(
            TCellAgent::PolicyTypes::LFI
          ).and_return(@local_files_policy)
          expect(@local_files_policy).to receive(:block_file_access?).and_return(false)
          expect(TCellAgent::Cmdi).not_to receive(:parse_command_from_open)
        end

        after :each do
          expect(File.stat(@new_file_name).mode.to_s(8)[3..5]).to eq('755')
          expect(@result.binmode?).to eq true

          File.delete(NEW_FILE_NAME) if File.exist?(NEW_FILE_NAME)
        end

        test_ruby2_ruby3_keywords(Kernel,
                                  'open',
                                  [NEW_FILE_NAME, 'w', 0o755],
                                  { :binmode => true },
                                  nil)

        test_ruby2_ruby3_keywords(Object,
                                  'open',
                                  [NEW_FILE_NAME, 'w', 0o755],
                                  { :binmode => true },
                                  nil)
      end
    end
    context 'with filename blocked for read/write' do
      before do |test|
        unless test.metadata[:skip_before]
          expect(TCellAgent).to receive(:policy).with(
            TCellAgent::PolicyTypes::LFI
          ).and_return(@local_files_policy, @local_files_policy)
          expect(@local_files_policy).to receive(:block_file_access?).and_return(true, true)
          expect(TCellAgent::Cmdi).not_to receive(:parse_command_from_open)
        end
      end

      it 'executes OS commands', :skip_before do
        result = Kernel.open('|echo test').read
        expect(result).to eq "test\n"

        result = open('|echo test').read
        expect(result).to eq "test\n"
      end
      it 'raises an IOError' do
        expect do
          Kernel.open(@new_file_name, 'w')
        end.to raise_error(IOError)

        expect do
          open(@new_file_name, 'w')
        end.to raise_error(IOError)
      end
      it 'raises an IOError' do
        expect do
          Kernel.open(@new_file_name, 'w')
        end.to raise_error(IOError)

        expect do
          open(@new_file_name, 'w')
        end.to raise_error(IOError)
      end
      it 'raises an IOError' do
        expect do
          Kernel.open(@new_file_name, 'a')
        end.to raise_error(IOError)

        expect do
          open(@new_file_name, 'a')
        end.to raise_error(IOError)
      end
    end
  end

  describe '::gets and #gets' do
    context 'with a filename not blocked for read/write' do
      it 'should still be able to read the file' do
        expect(TCellAgent).to receive(:policy).with(
          TCellAgent::PolicyTypes::LFI
        ).and_return(@local_files_policy, @local_files_policy, @local_files_policy, @local_files_policy)
        expect(@local_files_policy).to receive(:block_file_access?).and_return(false, false, false, false)

        copy = Array.new(ARGV)
        ARGV.clear
        ARGV << @filename
        result = gets
        result << gets
        ARGV.replace(copy)
        expect(result).to eq @file_contents

        copy = Array.new(ARGV)
        ARGV.clear
        ARGV << @filename
        result = Kernel.gets
        result << Kernel.gets
        ARGV.replace(copy)
        expect(result).to eq @file_contents
      end
    end
    context 'with a filename blocked for read/write' do
      it 'raises an IOError' do
        expect(TCellAgent).to receive(:policy).with(
          TCellAgent::PolicyTypes::LFI
        ).and_return(@local_files_policy, @local_files_policy)
        expect(@local_files_policy).to receive(:block_file_access?).and_return(true, true)

        copy = Array.new(ARGV)
        ARGV.clear
        ARGV << @filename
        expect do
          result = gets
        end.to raise_error(IOError)
        ARGV.replace(copy)

        copy = Array.new(ARGV)
        ARGV.clear
        ARGV << @filename
        expect do
          result = Kernel.gets
        end.to raise_error(IOError)
        ARGV.replace(copy)
      end
    end
  end

  describe '::readline and #readline' do
    context 'with a filename not blocked for read/write' do
      it 'reads the file' do
        expect(TCellAgent).to receive(:policy).with(
          TCellAgent::PolicyTypes::LFI
        ).and_return(@local_files_policy, @local_files_policy, @local_files_policy, @local_files_policy)
        expect(@local_files_policy).to receive(:block_file_access?).and_return(false, false, false, false)

        copy = Array.new(ARGV)
        ARGV.clear
        ARGV << @filename
        result = readline
        result << readline
        ARGV.replace(copy)
        expect(result).to eq @file_contents

        result = ''
        copy = Array.new(ARGV)
        ARGV.clear
        ARGV << @filename
        result = Kernel.readline
        result << Kernel.readline
        ARGV.replace(copy)
        expect(result).to eq @file_contents
      end
    end
    context 'with a filename blocked for read' do
      it 'raises an IOError' do
        expect(TCellAgent).to receive(:policy).with(
          TCellAgent::PolicyTypes::LFI
        ).and_return(@local_files_policy, @local_files_policy)
        expect(@local_files_policy).to receive(:block_file_access?).and_return(true, true)

        expect do
          copy = Array.new(ARGV)
          ARGV.clear
          ARGV << @filename
          Kernel.readline
          ARGV.replace(copy)
        end.to raise_error(IOError)
        expect do
          copy = Array.new(ARGV)
          ARGV.clear
          ARGV << @filename
          Kernel.readline
          ARGV.replace(copy)
        end.to raise_error(IOError)
      end
    end
  end
end
# rubocop:enable Security/Open
# rubocop:enable Lint/UselessAssignment
