require 'spec_helper'

module TCellAgent
  module Instrumentation
    module Lfi
      describe '.extract_path_mode' do
        context 'with path' do
          it 'should extract the path correctly' do
            args = '/path-to-file'
            path, _mode = TCellAgent::Instrumentation::Lfi.extract_path_mode(args)
            expect(path).to eq(File.expand_path('/path-to-file'))
          end
          it 'should default to read mode' do
            args = '/path-to-file'
            _path, mode = TCellAgent::Instrumentation::Lfi.extract_path_mode(args)
            expect(mode).to eq('Read')
          end
          it 'should not expand paths starting with |' do
            args = '|cat /etc/passwd'
            path, mode = TCellAgent::Instrumentation::Lfi.extract_path_mode(args)
            expect(path).to eq('')
            expect(mode).to eq('')
          end
          it 'should not expand paths starting with |-' do
            args = '|-'
            path, mode = TCellAgent::Instrumentation::Lfi.extract_path_mode(args)
            expect(path).to eq('')
            expect(mode).to eq('')
          end
        end
        context 'with path and mode' do
          context 'with a valid mode' do
            it 'should return Read for mode r' do
              args = '/path-to-file', 'r'
              _path, mode = TCellAgent::Instrumentation::Lfi.extract_path_mode(*args)
              expect(mode).to eq('Read')

              args = '/path-to-file', { :mode => 'r' }
              _path, mode = TCellAgent::Instrumentation::Lfi.extract_path_mode(*args)
              expect(mode).to eq('Read')
            end
            it 'should return Write for mode w' do
              args = '/path-to-file', 'w'
              _path, mode = TCellAgent::Instrumentation::Lfi.extract_path_mode(*args)
              expect(mode).to eq('Write')

              args = '/path-to-file', { :mode => 'w' }
              _path, mode = TCellAgent::Instrumentation::Lfi.extract_path_mode(*args)
              expect(mode).to eq('Write')
            end
            it 'should return Write for mode a' do
              args = '/path-to-file', 'a'
              _path, mode = TCellAgent::Instrumentation::Lfi.extract_path_mode(*args)
              expect(mode).to eq('Write')

              args = '/path-to-file', { :mode => 'a' }
              _path, mode = TCellAgent::Instrumentation::Lfi.extract_path_mode(*args)
              expect(mode).to eq('Write')
            end
            it 'should return ReadWrite for mode r+' do
              args = '/path-to-file', 'r+'
              _path, mode = TCellAgent::Instrumentation::Lfi.extract_path_mode(*args)
              expect(mode).to eq('ReadWrite')

              args = '/path-to-file', { :mode => 'r+' }
              _path, mode = TCellAgent::Instrumentation::Lfi.extract_path_mode(*args)
              expect(mode).to eq('ReadWrite')
            end
            it 'should return ReadWrite for mode w+' do
              args = '/path-to-file', 'w+'
              _path, mode = TCellAgent::Instrumentation::Lfi.extract_path_mode(*args)
              expect(mode).to eq('ReadWrite')

              args = '/path-to-file', { :mode => 'w+' }
              _path, mode = TCellAgent::Instrumentation::Lfi.extract_path_mode(*args)
              expect(mode).to eq('ReadWrite')
            end
            it 'should return ReadWrite for mode a+' do
              args = '/path-to-file', 'a+'
              _path, mode = TCellAgent::Instrumentation::Lfi.extract_path_mode(*args)
              expect(mode).to eq('ReadWrite')

              args = '/path-to-file', { :mode => 'a+' }
              _path, mode = TCellAgent::Instrumentation::Lfi.extract_path_mode(*args)
              expect(mode).to eq('ReadWrite')
            end
            it 'should return Read for mode ::File::RDONLY (0)' do
              args = '/path-to-file', ::File::RDONLY
              _path, mode = TCellAgent::Instrumentation::Lfi.extract_path_mode(*args)
              expect(mode).to eq('Read')

              args = '/path-to-file', { :mode => ::File::RDONLY }
              _path, mode = TCellAgent::Instrumentation::Lfi.extract_path_mode(*args)
              expect(mode).to eq('Read')
            end
            it 'should return Write for mode ::File::WRONLY (1)' do
              args = '/path-to-file', ::File::WRONLY
              _path, mode = TCellAgent::Instrumentation::Lfi.extract_path_mode(*args)
              expect(mode).to eq('Write')

              args = '/path-to-file', { :mode => ::File::WRONLY }
              _path, mode = TCellAgent::Instrumentation::Lfi.extract_path_mode(*args)
              expect(mode).to eq('Write')
            end
            it 'should return ReadWrite for mode ::File::RDWR (2)' do
              args = '/path-to-file', ::File::RDWR
              _path, mode = TCellAgent::Instrumentation::Lfi.extract_path_mode(*args)
              expect(mode).to eq('ReadWrite')

              args = '/path-to-file', { :mode => ::File::RDWR }
              _path, mode = TCellAgent::Instrumentation::Lfi.extract_path_mode(*args)
              expect(mode).to eq('ReadWrite')
            end
            it 'should return Write for mode ::File::CREAT | ::File::EXCL | ::File::WRONLY (2561)' do
              args = '/path-to-file', ::File::CREAT | ::File::EXCL | ::File::WRONLY
              _path, mode = TCellAgent::Instrumentation::Lfi.extract_path_mode(*args)
              expect(mode).to eq('Write')

              args = '/path-to-file', { :mode => ::File::CREAT | ::File::EXCL | ::File::WRONLY }
              _path, mode = TCellAgent::Instrumentation::Lfi.extract_path_mode(*args)
              expect(mode).to eq('Write')
            end
          end
          context 'with an invalid mode' do
            it 'should return Read when mode is a random string' do
              args = '/path-to-file', 'mode'
              _path, mode = TCellAgent::Instrumentation::Lfi.extract_path_mode(*args)
              expect(mode).to eq('Read')
            end
            it 'should return Read when mode is an empty hash' do
              args = '/path-to-file', {}
              _path, mode = TCellAgent::Instrumentation::Lfi.extract_path_mode(*args)
              expect(mode).to eq('Read')
            end
            it 'should return Read when mode is a hash without a :mode key' do
              args = '/path-to-file', { :placeholder => 'testing' }
              _path, mode = TCellAgent::Instrumentation::Lfi.extract_path_mode(*args)
              expect(mode).to eq('Read')
            end
            it 'should return Read when mode is an array' do
              args = '/path-to-file', []
              _path, mode = TCellAgent::Instrumentation::Lfi.extract_path_mode(*args)
              expect(mode).to eq('Read')
            end
          end
        end
      end

      describe '.raise_if_block' do
        context 'when passed a blocked path' do
          it 'raises an error' do
            expect(TCellAgent::Instrumentation::Lfi).to receive(:block_file_access?).with(
              '/blocked', 'Read'
            ).and_return(true)

            expect do
              TCellAgent::Instrumentation::Lfi.raise_if_block('/blocked', 'Read')
            end.to raise_error(IOError)
          end
        end
        context 'when passed a path not blocked' do
          it 'returns nil' do
            expect(TCellAgent::Instrumentation::Lfi).to receive(:block_file_access?).with(
              '/not-blocked', 'Read'
            ).and_return(false)

            expect(TCellAgent::Instrumentation::Lfi.raise_if_block('/not-blocked', 'Read')).to eq nil
          end
        end
      end

      describe '.default_open_handler' do
        it 'calls .raise_if_block' do
          expect(TCellAgent::Instrumentation::Lfi).to receive(:raise_if_block).with(
            '/placeholder', 'Read'
          ).and_return(nil)

          expect(TCellAgent::Instrumentation::Lfi.default_open_handler(['/placeholder'], 'Read')).to eq nil
        end

        it 'replaces the mode with override_mode' do
          expect(TCellAgent::Instrumentation::Lfi).to receive(:extract_path_mode).with(
            '/placeholder'
          ).and_return(['/placeholder', 'Read'])
          expect(TCellAgent::Instrumentation::Lfi).to receive(:raise_if_block).with(
            '/placeholder', 'ReadWrite'
          ).and_return(nil)

          expect(TCellAgent::Instrumentation::Lfi.default_open_handler(['/placeholder'], 'ReadWrite')).to eq nil
        end
      end

      describe '.argf_open_handler' do
        it 'calls .extract_path_mode_argf' do
          expect(TCellAgent::Instrumentation::Lfi).to receive(:extract_path_mode_argf).and_return(
            ['/placeholder', 'Read']
          )

          expect(TCellAgent::Instrumentation::Lfi.argf_open_handler).to eq nil
        end
      end
      describe '.cmdi_open_handler' do
        it 'behaves the similarly to default_open_handler' do
          expect(TCellAgent::Instrumentation::Lfi).to receive(:raise_if_block).with(
            '/placeholder', 'Read'
          ).and_return(nil)

          expect(TCellAgent::Instrumentation::Lfi.default_open_handler(['/placeholder'], 'Read')).to eq nil
        end

        it 'raises an error if command is blocked' do
          expect(TCellAgent::Cmdi).to receive(:block_command?).with(
            'ls'
          ).and_return(true)

          expect do
            TCellAgent::Instrumentation::Lfi.cmdi_open_handler('|ls')
          end.to raise_error(RuntimeError)
        end
      end
    end
  end
end
