require 'spec_helper'

module TCellAgent
  module Instrumentation
    describe '.block?' do
      before(:each) do
        native_agent = double('native_agent')
        @patches_policy = TCellAgent::Policies::PatchesPolicy.new(
          native_agent,
          {}
        )
      end

      context 'with an unexpected error' do
        it 'should return false' do
          logger = double('logger')
          request = double('request')
          expect(TCellAgent).to receive(:policy).with(
            TCellAgent::PolicyTypes::PATCHES
          ).and_raise(StandardError.new('UNEXPECTED'))
          expect(TCellAgent::MetaData).to_not receive(:for_patches)
          expect(TCellAgent::Instrumentation).to receive(:get_safe_block_logger).and_return(logger)
          expect(logger).to receive(:error).with(
            'Error Checking patches blocking (StandardError): UNEXPECTED'
          )
          expect(logger).to receive(:exception) # exception stack trace

          expect(Patches.block?(request)).to eq(false)
        end
      end

      context 'with a disabled patches policy' do
        it 'should return false' do
          request = double('request')
          expect(@patches_policy.enabled).to eq(false)

          expect(TCellAgent).to receive(:policy).and_return(@patches_policy)
          expect(TCellAgent::MetaData).to_not receive(:for_patches)

          expect(Patches.block?(request)).to eq(false)
        end
      end

      context 'with a patches policy that does not apply' do
        it 'should return false' do
          request = double('request')
          meta_data = double('meta_data')
          tcell_context = TCellAgent::Instrumentation::TCellData.new

          expect(TCellAgent).to receive(:policy).and_return(@patches_policy)
          expect(@patches_policy).to receive(:enabled).and_return(true)
          expect(@patches_policy).to receive(:block_request?).and_return(false)
          expect(request).to receive(:env).and_return(
            {
              TCellAgent::Instrumentation::TCELL_ID => tcell_context
            }
          )
          expect(TCellAgent::MetaData).to receive(:for_patches).and_return(
            meta_data
          )

          expect(Patches.block?(request)).to eq(false)
          expect(tcell_context.patches_blocking_triggered).to eq(false)
        end
      end

      context 'with a patches policy that applies' do
        it 'should return a response' do
          request = double('request')
          meta_data = double('meta_data')
          tcell_context = TCellAgent::Instrumentation::TCellData.new
          expect(tcell_context.patches_blocking_triggered).to eq(false)

          expect(TCellAgent).to receive(:policy).and_return(@patches_policy)
          expect(@patches_policy).to receive(:enabled).and_return(true)
          expect(@patches_policy).to receive(:block_request?).and_return(true)
          expect(TCellAgent::MetaData).to receive(:for_patches).and_return(
            meta_data
          )
          expect(request).to receive(:env).and_return({ TCellAgent::Instrumentation::TCELL_ID => tcell_context })

          expect(Patches.block?(request)).to eq(true)

          expect(tcell_context.patches_blocking_triggered).to eq(true)
        end

        context "and that's complex" do
          it 'should return a response' do
            request = double('request')
            meta_data = TCellAgent::MetaData.new(
              'get',
              '2.3.4.5',
              'route_id',
              'session_id',
              'user_id',
              'transaction_id',
              'http://test.com/',
              '0.0.0.0'
            )
            meta_data.get_dict = { 'paramater' => '<script>' }
            tcell_context = TCellAgent::Instrumentation::TCellData.new

            expect(tcell_context.patches_blocking_triggered).to eq(false)

            expect(TCellAgent).to receive(:policy).and_return(@patches_policy)
            expect(@patches_policy).to receive(:enabled).and_return(true)
            expect(@patches_policy).to receive(:block_request?).and_return(true)
            expect(TCellAgent::MetaData).to receive(:for_patches).and_return(
              meta_data
            )
            expect(request).to receive(:env).and_return({ TCellAgent::Instrumentation::TCELL_ID => tcell_context })

            expect(Patches.block?(request)).to eq(true)

            expect(tcell_context.patches_blocking_triggered).to eq(true)
          end
        end
      end
    end
  end
end
