require 'spec_helper'

module TCellAgent
  module Policies
    describe AppfirewallPolicy do
      everything_enabled_policy_json = {
        'appsensor' => {
          'policy_id' => '01a1',
          'version' => 2,
          'data' => {
            'options' => {
              'uri_options' => {
                'collect_full_uri' => true
              },
              'payloads' => {
                'send_payloads' => true,
                'send_blacklist' => {
                  'ssn' => ['*'],
                  'password' => ['*']
                },
                'send_whitelist' => {},
                'log_payloads' => true,
                'log_blacklist' => {},
                'log_whitelist' => {
                  'username' => ['*']
                }
              }
            },
            'sensors' => {
              'req_size' => {
                'limit' => 1024,
                'exclude_routes' => ['2300']
              },
              'resp_size' => {
                'limit' => 2048,
                'exclude_routes' => ['2323']
              },
              'resp_codes' => {
                'series_400_enabled' => true,
                'series_500_enabled' => true
              },
              'xss' => {
                'libinjection' => true,
                'patterns' => %w[1 2 8],
                'exclusions' => {
                  'bob' => ['*']
                }
              },
              'sqli' => {
                'libinjection' => true,
                'exclude_headers' => true,
                'patterns' => ['1']
              },
              'fpt' => {
                'patterns' => %w[1 2],
                'exclude_forms' => true,
                'exclude_cookies' => true,
                'exclusions' => {
                  'somethingcommon' => ['form']
                }
              },
              'cmdi' => {
                'patterns' => %w[1 2]
              },
              'nullbyte' => {
                'patterns' => %w[1 2]
              },
              'retr' => {
                'patterns' => %w[1 2]
              },
              'ua' => {
                'empty_enabled' => true
              },
              'errors' => {
                'csrf_exception_enabled' => true,
                'sql_exception_enabled' => true
              },
              'database' => {
                'large_result' => {
                  'limit' => 10
                }
              }
            }
          }
        },
        'regex' => {
          'data' => {
            'patterns' => [
              {
                'id' => 'tc-xss-1',
                'pattern' => '(?:<(script))',
                'sensor' => 'xss',
                'title' => 'Basic Injection'
              },
              {
                'safe_pattern' => '^[a-zA-Z0-9_\\s\\r\\n\\t]*$',
                'pattern' => '(?:[\\s()]case\\s*\\()|(?:\\)\\s*like\\s*\\()|(?:having\\s*[^\\s]+\\s*[^\\w\\s])|(?:if\\s?\\([\\d\\w]\\s*[=<>~])',
                'sensor' => 'sqli',
                'id' => 'tc-sqli-1',
                'title' => 'Conditional Attempts'
              }
            ],
            'version' => 1_518_546_622_571
          },
          'policy_id' => 'f3a313b0-10eb-11e8-8080-808080808080',
          'version' => 1
        }
      }

      describe '#initialize' do
        context 'empty enablements' do
          it 'should disable the policy' do
            native_agent = double('native_agent')
            policy = AppfirewallPolicy.new(native_agent, {})
            expect(policy.enabled).to eq(false)
          end
        end
      end

      describe '#check_appfirewall_injections' do
        context 'with disabled policy' do
          it 'should not call the native library' do
            native_agent = double('native_agent')
            policy = AppfirewallPolicy.new(native_agent, {})

            expect(native_agent).to_not receive(:apply_appfirewall)

            appsensor_meta = double('appsensor_meta')
            policy.check_appfirewall_injections(appsensor_meta)
          end
        end

        context 'with enabled policy' do
          before(:each) do
            configuration = TCellAgent::Tests::ConfigurationBuilder.new.build
            @native_agent = TCellAgent::Rust::NativeAgent.create_agent(
              configuration
            )
            enablements = @native_agent.update_policies(
              everything_enabled_policy_json
            )['enablements']
            expect(enablements['appfirewall']).to eq(true)

            @policy = AppfirewallPolicy.new(@native_agent, enablements)
            expect(@policy.enabled).to eq(true)
          end

          after(:each) do
            TCellAgent::Rust::NativeAgent.free_agent(@native_agent.agent_ptr)
          end

          it 'should be able to call native library' do
            meta_data = TCellAgent::Tests::MetaDataBuilder.new.build
            resp = @policy.check_appfirewall_injections(meta_data)
            expect(resp).to eq({})
          end

          context 'exception raised in native_agent' do
            it 'should log the exception' do
              expected_error = StandardError.new('UNEXPECTED')
              logger = double('logger')
              expect(TCellAgent::Instrumentation).to receive(:get_safe_block_logger).and_return(logger)
              expect(logger).to receive(:error).with(
                'Error AppFirewall inspection (StandardError): UNEXPECTED'
              )
              expect(logger).to receive(:exception).with(
                expected_error
              )

              meta_data = TCellAgent::Tests::MetaDataBuilder.new.build
              expect(@native_agent).to receive(:apply_appfirewall).with(
                meta_data
              ).and_raise(expected_error)

              resp = @policy.check_appfirewall_injections(meta_data)
              expect(resp).to eq(nil)
            end
          end
        end
      end
    end
  end
end
