require 'spec_helper'

module TCellAgent
  module Policies
    describe CommandInjectionPolicy do
      describe '#initialize' do
        context 'empty enablements' do
          it 'should disable the policy' do
            native_agent = double('native_agent')
            policy = CommandInjectionPolicy.new(native_agent, {})
            expect(policy.enabled).to eq(false)
          end
        end
      end

      describe '#block_command?' do
        context 'with disabled policy' do
          it 'should not call the native library' do
            native_agent = double('native_agent')
            policy = CommandInjectionPolicy.new(native_agent, {})

            expect(native_agent).to_not receive(:apply_cmdi)

            tcell_context = double('tcell_context')
            expect(
              policy.block_command?('cat /etc/passwd && grep root', tcell_context)
            ).to eq(false)
          end
        end

        context 'with policies' do
          before(:each) do
            configuration = TCellAgent::Tests::ConfigurationBuilder.new.build
            @native_agent = TCellAgent::Rust::NativeAgent.create_agent(
              configuration
            )
            @tcell_context = TCellAgent::Tests::TCellContextBuilder.new.build
          end

          after(:each) do
            TCellAgent::Rust::NativeAgent.free_agent(@native_agent.agent_ptr)
          end

          context 'empty command rules policy' do
            it 'should disable policy' do
              enablements = @native_agent.update_policies(
                {
                  'cmdi' => {
                    'policy_id' => 'policy_id',
                    'version' => 1,
                    'data' => { 'command_rules' => [] }
                  }
                }
              )['enablements']
              expect(enablements['cmdi']).to eq(false)

              @policy = CommandInjectionPolicy.new(@native_agent, enablements)
              expect(@policy.enabled).to eq(false)
            end
          end

          context 'ignore all command rules' do
            it 'should disable policy' do
              enablements = @native_agent.update_policies(
                {
                  'cmdi' => {
                    'policy_id' => 'policy_id',
                    'version' => 1,
                    'data' => {
                      'command_rules' => [{ 'rule_id' => '1', 'action' => 'ignore' }]
                    }
                  }
                }
              )['enablements']
              expect(enablements['cmdi']).to eq(false)

              @policy = CommandInjectionPolicy.new(@native_agent, enablements)
              expect(@policy.enabled).to eq(false)
            end
          end

          context 'report all command rules' do
            it 'should enable the policy but not block command' do
              enablements = @native_agent.update_policies(
                {
                  'cmdi' => {
                    'policy_id' => 'policy_id',
                    'version' => 1,
                    'data' => {
                      'collect_full_commandline' => true,
                      'command_rules' => [{ 'rule_id' => '1', 'action' => 'report' }]
                    }
                  }
                }
              )['enablements']
              expect(enablements['cmdi']).to eq(true)

              @policy = CommandInjectionPolicy.new(@native_agent, enablements)
              expect(@policy.enabled).to eq(true)
              expect(
                @policy.block_command?('cat /etc/passwd && grep root', @tcell_context)
              ).to eq(false)
            end
          end

          context 'block all command rules' do
            it 'should block the command' do
              enablements = @native_agent.update_policies(
                {
                  'cmdi' => {
                    'policy_id' => 'policy_id',
                    'version' => 1,
                    'data' => {
                      'collect_full_commandline' => true,
                      'command_rules' => [{ 'rule_id' => '1', 'action' => 'block' }]
                    }
                  }
                }
              )['enablements']
              expect(enablements['cmdi']).to eq(true)

              @policy = CommandInjectionPolicy.new(@native_agent, enablements)
              expect(@policy.enabled).to eq(true)
              expect(
                @policy.block_command?('cat /etc/passwd && grep root', @tcell_context)
              ).to eq(true)
            end
          end

          context 'report compound commands' do
            it 'should enable the policy but no block the command' do
              enablements = @native_agent.update_policies(
                {
                  'cmdi' => {
                    'policy_id' => 'policy_id',
                    'version' => 1,
                    'data' => {
                      'compound_statement_rules' => [{ 'rule_id' => '1', 'action' => 'report' }]
                    }
                  }
                }
              )['enablements']
              expect(enablements['cmdi']).to eq(true)

              @policy = CommandInjectionPolicy.new(@native_agent, enablements)
              expect(@policy.enabled).to eq(true)

              expect(
                @policy.block_command?('cat /etc/passwd', @tcell_context)
              ).to eq(false)

              expect(
                @policy.block_command?('cat /etc/passwd && grep root', @tcell_context)
              ).to eq(false)
            end
          end

          context 'block compound commands' do
            it 'should enable the policy and block compound commands' do
              enablements = @native_agent.update_policies(
                {
                  'cmdi' => {
                    'policy_id' => 'policy_id',
                    'version' => 1,
                    'data' => {
                      'compound_statement_rules' => [{ 'rule_id' => '1', 'action' => 'block' }]
                    }
                  }
                }
              )['enablements']
              expect(enablements['cmdi']).to eq(true)

              @policy = CommandInjectionPolicy.new(@native_agent, enablements)
              expect(@policy.enabled).to eq(true)

              expect(
                @policy.block_command?('cat /etc/passwd', @tcell_context)
              ).to eq(false)

              expect(
                @policy.block_command?('cat /etc/passwd && grep root', @tcell_context)
              ).to eq(true)
            end
          end
        end
      end
    end
  end
end
