require 'spec_helper'

module TCellAgent
  module Policies
    describe HeadersPolicy do
      describe '#initialize' do
        context 'empty enablements' do
          it 'should disable the policy' do
            native_agent = double('native_agent')
            policy = HeadersPolicy.new(native_agent, {})
            expect(policy.enabled).to eq(false)
          end
        end
      end

      describe '#get_headers' do
        context 'with disabled policy' do
          it 'should not call the native library' do
            native_agent = double('native_agent')
            policy = HeadersPolicy.new(native_agent, {})

            expect(native_agent).to_not receive(:get_headers)

            tcell_context = double('tcell_context')
            policy.get_headers('text/html', tcell_context)
          end
        end

        context 'with enabled policy' do
          before(:each) do
            configuration = TCellAgent::Tests::ConfigurationBuilder.new.update_attribute(
              'max_csp_header_bytes', 105
            ).build
            @native_agent = TCellAgent::Rust::NativeAgent.create_agent(
              configuration
            )
            @tcell_context = TCellAgent::Tests::TCellContextBuilder.new.update_attribute(
              'session_id', 'session-id'
            ).update_attribute(
              'route_id', 'route-id'
            ).build
          end

          after(:each) do
            TCellAgent::Rust::NativeAgent.free_agent(@native_agent.agent_ptr)
          end

          it 'should return csp header' do
            enablements = @native_agent.update_policies(
              {
                'csp-headers' => {
                  'version' => 1,
                  'policy_id' => 'xyzd',
                  'headers' => [
                    { 'name' => 'Content-Security-Policy',
                      'value' => 'test321' }
                  ]
                }
              }
            )['enablements']
            expect(enablements['headers']).to eq(true)

            @policy = HeadersPolicy.new(@native_agent, enablements)
            expect(@policy.enabled).to eq(true)

            expect(
              @policy.get_headers('text/html', @tcell_context)
            ).to eq(
              [{ 'name' => 'Content-Security-Policy', 'value' => 'test321' }]
            )
          end

          context 'header with report uri' do
            it 'should return csp header' do
              enablements = @native_agent.update_policies(
                {
                  'csp-headers' => {
                    'version' => 1,
                    'policy_id' => 'xyzd',
                    'headers' => [
                      { 'name' => 'Content-Security-Policy',
                        'value' => 'normalvalue',
                        'report_uri' => 'https://www.example.com/xys' }
                    ]
                  }
                }
              )['enablements']
              expect(enablements['headers']).to eq(true)

              @policy = HeadersPolicy.new(@native_agent, enablements)
              expect(@policy.enabled).to eq(true)

              expect(
                @policy.get_headers('text/html', @tcell_context)
              ).to eq(
                [{ 'name' => 'Content-Security-Policy',
                   'value' => 'normalvalue; report-uri https://www.example.com/xys?sid=ab7074d0bf86c2884766d88b6ad9de4a&rid=route-id' }]
              )
            end
          end

          context 'header equal to max csp header bytes' do
            it 'should return csp header' do
              enablements = @native_agent.update_policies(
                {
                  'csp-headers' => {
                    'version' => 1,
                    'policy_id' => 'xyzd',
                    'headers' => [
                      { 'name' => 'Content-Security-Policy',
                        'value' => 'normalvalue',
                        'report_uri' => 'https://www.example.com/1234567' }
                    ]
                  }
                }
              )['enablements']
              expect(enablements['headers']).to eq(true)

              @policy = HeadersPolicy.new(@native_agent, enablements)
              expect(@policy.enabled).to eq(true)

              expect(
                @policy.get_headers('text/html', @tcell_context)
              ).to eq(
                [{ 'name' => 'Content-Security-Policy',
                   'value' => 'normalvalue; report-uri https://www.example.com/1234567?sid=ab7074d0bf86c2884766d88b6ad9de4a&rid=route-id' }]
              )
            end
          end

          context 'header exceeding to max csp header bytes' do
            it 'should not return csp header' do
              enablements = @native_agent.update_policies(
                {
                  'csp-headers' => {
                    'version' => 1,
                    'policy_id' => 'xyzd',
                    'headers' => [
                      { 'name' => 'Content-Security-Policy',
                        'value' => 'normalvalue',
                        'report_uri' => 'https://www.example.com/12345678' }
                    ]
                  }
                }
              )['enablements']
              expect(enablements['headers']).to eq(true)

              @policy = HeadersPolicy.new(@native_agent, enablements)
              expect(@policy.enabled).to eq(true)

              expect(
                @policy.get_headers('text/html', @tcell_context)
              ).to eq([])
            end
          end
        end
      end
    end
  end
end
