require 'spec_helper'
require 'set'

module TCellAgent
  module Policies
    describe DataLossPolicy do
      policy_json_two = {
        'policy_id' => 'x1a1',
        'data' => {
          'session_id_protections' => { 'body' => ['redact'], 'log' => ['event'] }
        }
      }
      policy_two = DataLossPolicy.new(policy_json_two)
      context 'check session_id_protections' do
        it 'gives the right actions' do
          expect(policy_two.get_actions_for_session_id.body_redact).to eq(true)
          expect(policy_two.get_actions_for_session_id.log_redact).to eq(nil)
          expect(policy_two.get_actions_for_session_id.log_event).to eq(true)
        end
      end
      context 'Database Options' do
        it 'Ignores bad table' do
          policy_json_requests = {
            'policy_id' => 'x1a1',
            'data' => {
              'db_protections' => [
                {
                  'databases' => ['dave'],
                  'schemas' => ['sam'],
                  'tables' => ['trevor'],
                  'fields' => ['fred'],
                  'actions' => {
                    'log' => ['redact'],
                    'body' => ['event']
                  }
                }
              ]
            }
          }
          db_one_policy = DataLossPolicy.new(policy_json_requests)
          expect(db_one_policy.get_actions_for_table('dave', 'sam', 'tommy', 'fred')).to eq(nil)
        end
        it 'Partial Policy' do
          # Assume if databases, schemas that are missing are "*"
          policy_json_requests = {
            'policy_id' => 'x1a1',
            'data' => {
              'db_protections' => [
                {
                  'fields' => ['fred'],
                  'actions' => {
                    'log' => ['redact'],
                    'body' => ['event']
                  }
                }
              ]
            }
          }
          db_one_policy = DataLossPolicy.new(policy_json_requests)
          expect(db_one_policy.get_actions_for_table('dave', 'sam', 'tommy', 'fred').to_a[0].log_redact).to eq(true)
          expect(db_one_policy.get_actions_for_table('dave', 'sam', 'tommy', 'fred', 'abcd').to_a[0].log_redact).to eq(true)
        end
        it 'Scopes by reoute_id' do
          policy_json_request_ids = {
            'policy_id' => 'x1a1',
            'data' => {
              'db_protections' => [
                {
                  'scope' => 'route',
                  'route_ids' => ['abcd'],
                  'databases' => ['dave'],
                  'schemas' => ['sam'],
                  'tables' => ['tommy'],
                  'fields' => ['fred'],
                  'actions' => {
                    'log' => ['redact'],
                    'body' => ['event']
                  }
                }
              ]
            }
          }
          db_two_policy = DataLossPolicy.new(policy_json_request_ids)
          expect(db_two_policy.get_actions_for_table('dave', 'sam', 'tommy', 'fred').to_a.size).to eq(0)
          expect(db_two_policy.get_actions_for_table('dave', 'sam', 'tommy', 'fred', 'other_route').to_a.size).to eq(0)
          expect(db_two_policy.get_actions_for_table('dave', 'sam', 'tommy', 'fred', 'abcd').to_a.size).to eq(1)

          expect(db_two_policy.get_actions_for_table('dave', 'sam', 'tommy', 'fred', 'abcd').to_a[0].log_redact).to eq(true)
          expect(db_two_policy.get_actions_for_table('dave', 'sam', 'tommy', 'fred', 'abcd').to_a[0].body_redact).to eq(nil)
          expect(db_two_policy.get_actions_for_table('dave', 'sam', 'tommy', 'fred', 'abcd').to_a[0].body_event).to eq(true)
        end
      end
      context 'Request Options' do
        it 'Ignores non-global scoped policy' do
          policy_json_requests = {
            'policy_id' => 'x1a1',
            'data' => {
              'request_protections' => [
                {
                  'variable_context' => 'form',
                  'variables' => ['test123'],
                  'actions' => {
                    'log' => ['redact'],
                    'body' => ['event']
                  }
                }
              ]
            }
          }
          policy_three = DataLossPolicy.new(policy_json_requests)
          expect(policy_three.get_actions_for_request('form', 'test123').to_a[0].body_event).to eq(true)
          expect(policy_three.get_actions_for_request('form', 'test123').to_a[0].log_event).to eq(nil)
        end
        it 'Reads in the policy' do
          policy_json_requests = {
            'policy_id' => 'x1a1',
            'data' => {
              'request_protections' => [
                {
                  'variable_context' => 'form',
                  'scope' => 'route',
                  'route_ids' => ['routex'],
                  'variables' => ['test123'],
                  'actions' => {
                    'log' => ['redact'],
                    'body' => ['event']
                  }
                }
              ]
            }
          }
          policy_three = DataLossPolicy.new(policy_json_requests)
          entry_wildcard_route = policy_three.get_actions_for_request('form', 'TeSt123')
          entry_given_route = policy_three.get_actions_for_request('form', 'TeSt123', 'routex')
          expect(entry_wildcard_route).to eq(nil)
          expect(entry_given_route.size).to eq(1)
          expect(entry_given_route.to_a[0].body_redact).to eq(nil)
          expect(entry_given_route.to_a[0].log_redact).to eq(true)
        end

        it 'Reads in the policy with cookie (redact/event)' do
          policy_json_requests = {
            'policy_id' => 'x1a1',
            'data' => {
              'request_protections' => [
                {
                  'variable_context' => 'cookie',
                  'scope' => 'route',
                  'route_ids' => ['routex'],
                  'variables' => ['test123'],
                  'actions' => {
                    'log' => ['redact'],
                    'body' => ['event']
                  }
                }
              ]
            }
          }
          policy_three = DataLossPolicy.new(policy_json_requests)
          entry_wildcard_route = policy_three.get_actions_for_request('cookie', 'test123')
          entry_given_route = policy_three.get_actions_for_request('cookie', 'test123', 'routex')
          expect(entry_wildcard_route).to eq(nil)
          expect(entry_given_route.size).to eq(1)
          expect(entry_given_route.to_a[0].body_redact).to eq(nil)
          expect(entry_given_route.to_a[0].log_redact).to eq(true)
        end

        it 'Reads in the policy with mixed-case cookie (redact/event)' do
          policy_json_requests = {
            'policy_id' => 'x1a1',
            'data' => {
              'request_protections' => [
                {
                  'variable_context' => 'cookie',
                  'scope' => 'route',
                  'route_ids' => ['routex'],
                  'variables' => ['teST123'],
                  'actions' => {
                    'log' => ['redact'],
                    'body' => ['event']
                  }
                }
              ]
            }
          }
          policy_three = DataLossPolicy.new(policy_json_requests)
          entry_wildcard_route = policy_three.get_actions_for_request('cookie', 'test123')
          entry_given_route = policy_three.get_actions_for_request('cookie', 'test123', 'routex')
          expect(entry_wildcard_route).to eq(nil)
          expect(entry_given_route).to eq(nil)
        end

        it 'Reads in the policy with header (redact/event)' do
          policy_json_requests = {
            'policy_id' => 'x1a1',
            'data' => {
              'request_protections' => [
                {
                  'variable_context' => 'header',
                  'scope' => 'route',
                  'route_ids' => ['routex'],
                  'variables' => ['test123'],
                  'actions' => {
                    'log' => ['redact'],
                    'body' => ['event']
                  }
                }
              ]
            }
          }
          policy_three = DataLossPolicy.new(policy_json_requests)
          entry_wildcard_route = policy_three.get_actions_for_request('header', 'TeSt123')
          entry_given_route = policy_three.get_actions_for_request('header', 'TeSt123', 'routex')
          expect(entry_wildcard_route).to eq(nil)
          expect(entry_given_route.size).to eq(1)
          expect(entry_given_route.to_a[0].body_redact).to eq(nil)
          expect(entry_given_route.to_a[0].log_redact).to eq(true)
        end
      end
    end
  end
end
