require 'spec_helper'

module TCellAgent
  module Policies
    describe HttpRedirectPolicy do
      describe '#initialize' do
        context 'empty enablements' do
          it 'should disable the policy' do
            native_agent = double('native_agent')
            policy = HttpRedirectPolicy.new(native_agent, {})
            expect(policy.enabled).to eq(false)
          end
        end
      end

      describe '#check_redirect' do
        context 'with disabled policy' do
          it 'should not call the native library' do
            native_agent = double('native_agent')
            policy = HttpRedirectPolicy.new(native_agent, {})

            expect(native_agent).to_not receive(:check_http_redirect)

            tcell_context = double('tcell_context')
            expect(
              policy.check_redirect(
                'http://localhost:8011/abc/def',
                'localhost:8011',
                200,
                tcell_context
              )
            ).to eq('http://localhost:8011/abc/def')
          end
        end

        context 'with native agent' do
          before(:each) do
            configuration = TCellAgent::Tests::ConfigurationBuilder.new.build
            @native_agent = TCellAgent::Rust::NativeAgent.create_agent(
              configuration
            )
          end

          after(:each) do
            TCellAgent::Rust::NativeAgent.free_agent(@native_agent.agent_ptr)
          end

          context 'redirect to the same' do
            it 'should not block the redirect' do
              enablements = @native_agent.update_policies(
                {
                  'http-redirect' => {
                    'version' => 1,
                    'policy_id' => 'nyzd',
                    'data' => {
                      'enabled' => true,
                      'whitelist' => ['whitelisted'],
                      'block' => true
                    }
                  }
                }
              )['enablements']
              expect(enablements['http_redirect']).to eq(true)

              @policy = HttpRedirectPolicy.new(@native_agent, enablements)
              expect(@policy.enabled).to eq(true)

              tcell_context = TCellAgent::Tests::TCellContextBuilder.new.update_attribute(
                'path', '/etc/123'
              ).update_attribute(
                'remote_address', '0.1.1.0'
              ).update_attribute(
                'fullpath', '/etc/123'
              ).build
              check = @policy.check_redirect(
                'http://localhost:8011/abc/def',
                'localhost:8011',
                200,
                tcell_context
              )

              expect(check).to eq('http://localhost:8011/abc/def')
            end
          end

          context 'asterisk in whitelisted domain' do
            it 'should allow the redirect' do
              enablements = @native_agent.update_policies(
                {
                  'http-redirect' => {
                    'version' => 1,
                    'policy_id' => 'nyzd',
                    'data' => {
                      'enabled' => true,
                      'whitelist' => ['*.allowed*.com'],
                      'block' => true
                    }
                  }
                }
              )['enablements']
              expect(enablements['http_redirect']).to eq(true)

              @policy = HttpRedirectPolicy.new(@native_agent, enablements)
              expect(@policy.enabled).to eq(true)

              tcell_context = TCellAgent::Tests::TCellContextBuilder.new.update_attribute(
                'path', '/etc/123'
              ).update_attribute(
                'remote_address', '0.1.1.0'
              ).update_attribute(
                'fullpath', '/etc/123'
              ).build
              check = @policy.check_redirect(
                'http://allowed.com',
                'localhost:8011',
                200,
                tcell_context
              )

              expect(check).to eq('http://allowed.com')
            end
          end

          context 'domain with a port' do
            it 'should block the redirect' do
              enablements = @native_agent.update_policies(
                {
                  'http-redirect' => {
                    'version' => 1,
                    'policy_id' => 'nyzd',
                    'data' => {
                      'enabled' => true,
                      'whitelist' => [],
                      'block' => true
                    }
                  }
                }
              )['enablements']
              expect(enablements['http_redirect']).to eq(true)

              @policy = HttpRedirectPolicy.new(@native_agent, enablements)
              expect(@policy.enabled).to eq(true)

              tcell_context = TCellAgent::Tests::TCellContextBuilder.new.update_attribute(
                'path', '/some/path'
              ).update_attribute(
                'remote_address', '0.1.1.0'
              ).update_attribute(
                'fullpath', '/some/path'
              ).build
              check = @policy.check_redirect(
                'http://user:pass@192.168.99.100:3000',
                'localhost:8011',
                200,
                tcell_context
              )

              expect(check).to eq('/')
            end
          end

          context 'data scheme redirect' do
            context 'not allowed' do
              it 'should block the redirect' do
                enablements = @native_agent.update_policies(
                  {
                    'http-redirect' => {
                      'version' => 1,
                      'policy_id' => 'nyzd',
                      'data' => {
                        'enabled' => true,
                        'whitelist' => ['whitelisted'],
                        'block' => true,
                        'data_scheme_allowed' => false
                      }
                    }
                  }
                )['enablements']
                expect(enablements['http_redirect']).to eq(true)

                @policy = HttpRedirectPolicy.new(@native_agent, enablements)
                expect(@policy.enabled).to eq(true)

                tcell_context = TCellAgent::Tests::TCellContextBuilder.new.update_attribute(
                  'path', '/some/path'
                ).update_attribute(
                  'remote_address', '0.1.1.0'
                ).update_attribute(
                  'fullpath', '/some/path'
                ).build
                check = @policy.check_redirect(
                  'data:text/html base64,PHNjcmlwdD5hbGVydCgnWFNTJyk8L3NjcmlwdD4K',
                  'localhost:8011',
                  200,
                  tcell_context
                )

                expect(check).to eq('/')
              end
            end

            context 'is allowed' do
              it 'should allow the redirect' do
                enablements = @native_agent.update_policies(
                  {
                    'http-redirect' => {
                      'version' => 1,
                      'policy_id' => 'nyzd',
                      'data' => {
                        'enabled' => true,
                        'whitelist' => ['whitelisted'],
                        'block' => true,
                        'data_scheme_allowed' => true
                      }
                    }
                  }
                )['enablements']
                expect(enablements['http_redirect']).to eq(true)

                @policy = HttpRedirectPolicy.new(@native_agent, enablements)
                expect(@policy.enabled).to eq(true)

                tcell_context = TCellAgent::Tests::TCellContextBuilder.new.update_attribute(
                  'path', '/some/path'
                ).update_attribute(
                  'remote_address', '0.1.1.0'
                ).update_attribute(
                  'fullpath', '/some/path'
                ).build
                check = @policy.check_redirect(
                  'data:text/html base64,PHNjcmlwdD5hbGVydCgnWFNTJyk8L3NjcmlwdD4K',
                  'localhost:8011',
                  200,
                  tcell_context
                )

                expect(check).to eq('data:text/html base64,PHNjcmlwdD5hbGVydCgnWFNTJyk8L3NjcmlwdD4K')
              end
            end
          end

          context 'relative redirect' do
            it 'should allow the redirect' do
              enablements = @native_agent.update_policies(
                {
                  'http-redirect' => {
                    'version' => 1,
                    'policy_id' => 'nyzd',
                    'data' => {
                      'enabled' => true,
                      'whitelist' => [],
                      'block' => true
                    }
                  }
                }
              )['enablements']
              expect(enablements['http_redirect']).to eq(true)

              @policy = HttpRedirectPolicy.new(@native_agent, enablements)
              expect(@policy.enabled).to eq(true)

              tcell_context = TCellAgent::Tests::TCellContextBuilder.new.update_attribute(
                'path', '/some/path'
              ).update_attribute(
                'remote_address', '0.1.1.0'
              ).update_attribute(
                'fullpath', '/some/path'
              ).build
              check = @policy.check_redirect(
                '/admin/login/?next=/admin/',
                'localhost:8011',
                200,
                tcell_context
              )

              expect(check).to eq('/admin/login/?next=/admin/')
            end
          end
        end
      end
    end
  end
end
