require 'spec_helper'

module TCellAgent
  module Policies
    describe LoginPolicy do
      describe '#initialize' do
        context 'empty enablements' do
          it 'should disable the policy' do
            native_agent = double('native_agent')
            policy = LoginPolicy.new(native_agent, {})
            expect(policy.login_success_enabled).to eq(false)
            expect(policy.login_failed_enabled).to eq(false)
          end
        end
      end

      context 'with native agent' do
        before(:each) do
          configuration = TCellAgent::Tests::ConfigurationBuilder.new.build
          @native_agent = TCellAgent::Rust::NativeAgent.create_agent(
            configuration
          )
        end

        after(:each) do
          TCellAgent::Rust::NativeAgent.free_agent(@native_agent.agent_ptr)
        end

        describe '#report_login_success' do
          context 'with disabled policy' do
            it 'should not call the native library' do
              native_agent = double('native_agent')
              policy = LoginPolicy.new(native_agent, {})

              expect(native_agent).to_not receive(:login_fraud_apply)

              user_id = double('user_id')
              headers = double('headers')
              tcell_context = double('tcell_context')
              expect(
                policy.report_login_success(user_id, headers, tcell_context)
              ).to eq({})
            end
          end

          context 'with enabled policy' do
            it 'should report login success' do
              enablements = @native_agent.update_policies(
                {
                  'login' => {
                    'policy_id' => '00a1',
                    'version' => 1,
                    'data' => {
                      'options' => {
                        'login_failed_enabled' => true,
                        'login_success_enabled' => true
                      }
                    }
                  }
                }
              )['enablements']
              expect(enablements['login_success_enabled']).to eq(true)

              @policy = LoginPolicy.new(@native_agent, enablements)
              expect(@policy.login_success_enabled).to eq(true)

              user_id = 'user-id'
              headers = ['user-agent']
              tcell_context = TCellAgent::Tests::TCellContextBuilder.new.update_attribute(
                'user_id', 'user-id'
              ).build
              expect(
                @policy.report_login_success(user_id, headers, tcell_context)
              ).to eq(
                { 'events_created' => 1 }
              )
            end
          end
        end

        describe '#report_login_failure' do
          context 'with disabled policy' do
            it 'should not call the native library' do
              native_agent = double('native_agent')
              policy = LoginPolicy.new(native_agent, {})

              expect(native_agent).to_not receive(:login_fraud_apply)

              user_id = double('user_id')
              password = nil
              headers = double('headers')
              user_valid = false
              tcell_context = double('tcell_context')
              expect(
                policy.report_login_failure(
                  user_id,
                  password,
                  headers,
                  user_valid,
                  tcell_context
                )
              ).to eq({})
            end
          end

          context 'with enabled policy' do
            it 'should report login failure' do
              enablements = @native_agent.update_policies(
                {
                  'login' => {
                    'policy_id' => '00a1',
                    'version' => 1,
                    'data' => {
                      'options' => {
                        'login_failed_enabled' => true,
                        'login_success_enabled' => true
                      }
                    }
                  }
                }
              )['enablements']
              expect(enablements['login_failed_enabled']).to eq(true)

              @policy = LoginPolicy.new(@native_agent, enablements)
              expect(@policy.login_failed_enabled).to eq(true)

              user_id = 'user-id'
              password = nil
              headers = ['user-agent']
              user_valid = nil
              tcell_context = TCellAgent::Tests::TCellContextBuilder.new.update_attribute(
                'user_id', 'user-id'
              ).build
              expect(
                @policy.report_login_failure(
                  user_id,
                  password,
                  headers,
                  user_valid,
                  tcell_context
                )
              ).to eq(
                { 'events_created' => 1 }
              )
            end
          end
        end
      end
    end
  end
end

#    def login_failed_test(self):
#        policies_rsp = self.native_agent.update_policies({
#            "login": {
#                "policy_id": "00a1",
#                "version": 1,
#                "data": {
#                    "options": {
#                        "login_failed_enabled": True,
#                        "login_success_enabled": True
#                    }
#                }
#            }
#        })

#        login_policy = LoginPolicy(self.native_agent, policies_rsp["enablements"], None)
#        context = ContextBuilder().update_attribute("user_id", "user-id").build()
#        response = login_policy.report_login_failure(
#            user_id="user-id",
#            password="password",
#            header_keys=["user-agent", "content-length"],
#            user_valid=False,
#            tcell_context=context
#        )
#
#        self.assertEqual(response, {"events_created": 1})
