require 'spec_helper'

module TCellAgent
  module Policies
    describe PatchesPolicy do
      blocked_ip_policy = {
        'patches' => {
          'policy_id' => 'policy_id',
          'version' => 2,
          'data' => {
            'rules' => [
              {
                'id' => 'blocked-ips-rule',
                'title' => 'Blocked ips rule',
                'action' => 'BlockIf',
                'destinations' => { 'check_equals' => [{ 'path' => '*' }] },
                'ignore' => [],
                'matches' => [
                  {
                    'all' => [],
                    'any' => [
                      {
                        'ips' => [{ 'type' => 'IP', 'values' => ['1.1.1.1'] }]
                      }
                    ]
                  }
                ]
              }
            ]
          }
        }
      }

      describe '#initialize' do
        context 'empty enablements' do
          it 'should disable the policy' do
            native_agent = double('native_agent')
            policy = PatchesPolicy.new(native_agent, {})
            expect(policy.enabled).to eq(false)
          end
        end
      end

      describe '#block_request?' do
        context 'with disabled policy' do
          it 'should not call the native library' do
            native_agent = double('native_agent')
            policy = PatchesPolicy.new(native_agent, {})

            expect(native_agent).to_not receive(:apply_patches)

            appsensor_meta = double('appsensor_meta')
            policy.block_request?(appsensor_meta)
          end
        end

        context 'with blocked ip policy' do
          before(:each) do
            configuration = TCellAgent::Tests::ConfigurationBuilder.new.build
            @native_agent = TCellAgent::Rust::NativeAgent.create_agent(
              configuration
            )
            enablements = @native_agent.update_policies(
              blocked_ip_policy
            )['enablements']
            expect(enablements['patches']).to eq(true)

            @policy = PatchesPolicy.new(@native_agent, enablements)
            expect(@policy.enabled).to eq(true)
          end

          after(:each) do
            TCellAgent::Rust::NativeAgent.free_agent(@native_agent.agent_ptr)
          end

          context 'request has nil ip' do
            it 'should not block request' do
              meta_data = TCellAgent::Tests::MetaDataBuilder.new.update_attribute(
                'remote_address', nil
              ).build
              expect(@native_agent).to receive(:apply_suspicious_quick_check).with(any_args)
              expect(@native_agent).not_to receive(:apply_patches).with(any_args)
              resp = @policy.block_request?(meta_data)
              expect(resp).to eq(false)
            end
          end

          context 'request has empty ip' do
            it 'should not block request' do
              meta_data = TCellAgent::Tests::MetaDataBuilder.new.update_attribute(
                'remote_address', ''
              ).build
              expect(@native_agent).to receive(:apply_suspicious_quick_check).with(any_args)
              expect(@native_agent).not_to receive(:apply_patches).with(any_args)
              resp = @policy.block_request?(meta_data)
              expect(resp).to eq(false)
            end
          end

          context 'request comes from non-blocked ip' do
            it 'should not block request' do
              meta_data = TCellAgent::Tests::MetaDataBuilder.new.update_attribute(
                'remote_address', '2.2.2.2'
              ).build
              expect(@native_agent).to receive(:apply_suspicious_quick_check).with(any_args)
              expect(@native_agent).not_to receive(:apply_patches).with(any_args)
              resp = @policy.block_request?(meta_data)
              expect(resp).to eq(false)
            end
          end

          context 'request comes from blocked ip' do
            it 'should block request' do
              meta_data = TCellAgent::Tests::MetaDataBuilder.new.update_attribute(
                'remote_address', '1.1.1.1'
              ).build
              expect(@native_agent).to receive(:apply_suspicious_quick_check).with(any_args).and_return(2)
              expect(@native_agent).not_to receive(:apply_patches).with(any_args)
              resp = @policy.block_request?(meta_data)
              expect(resp).to eq(true)
            end
          end

          context 'request comes from suspcious ip' do
            it 'should call apply_patches' do
              meta_data = TCellAgent::Tests::MetaDataBuilder.new.update_attribute(
                'remote_address', '1.1.1.1'
              ).build
              expect(@native_agent).to receive(:apply_suspicious_quick_check).with(any_args).and_return(1)
              expect(@native_agent).to receive(:apply_patches).with(any_args).and_return('Blocked Response')
              @policy.block_request?(meta_data)
            end
          end
        end
      end
    end
  end
end
