require 'spec_helper'

module TCellAgent
  module Instrumentation
    module Rails
      describe '.insert_now' do
        context 'nil js_agent_handler' do
          it 'should do nothing' do
            script_insert = '<script>script_insert</script>'

            rack_body, content_legnth = JSAgent.insert_now(nil,
                                                           script_insert,
                                                           ['body'],
                                                           4)

            expect(rack_body).to eq(['body'])
            expect(content_legnth).to eq(4)
          end
        end

        context 'no content' do
          it 'should do nothing' do
            js_agent_handler = proc {
              raise Exception, 'should not be called'
            }
            script_insert = '<script>script_insert</script>'

            rack_body, content_legnth = JSAgent.insert_now(js_agent_handler,
                                                           script_insert,
                                                           [],
                                                           0)

            expect(rack_body).to eq([])
            expect(content_legnth).to eq(0)
          end
        end

        context 'body with 1 part' do
          context 'that does not match insert' do
            it 'should return orignal body and content length' do
              js_agent_handler = proc { |si, resp|
                resp.sub('REPLACE_ME', si)
              }
              script_insert = '<script>script_insert</script>'

              rack_body, content_legnth = JSAgent.insert_now(js_agent_handler,
                                                             script_insert,
                                                             ['i am the body'],
                                                             'i am the body'.bytesize)

              expect(rack_body).to eq(['i am the body'])
              expect(content_legnth).to eq(13)
            end
          end

          context 'that matches insert' do
            it 'should return modified body and new content length' do
              js_agent_handler = proc { |si, resp|
                resp.sub('REPLACE_ME', si)
              }
              script_insert = '<script>script_insert</script>'

              rack_body, content_legnth = JSAgent.insert_now(js_agent_handler,
                                                             script_insert,
                                                             ['i am REPLACE_ME the body'],
                                                             'i am REPLACE_ME the body'.bytesize)
              expect(rack_body).to eq(['i am <script>script_insert</script> the body'])
              expect(content_legnth).to eq(44)
            end
          end
        end

        context 'body with multiple parts' do
          context 'that does not match insert' do
            it 'should return orignal body and content length' do
              js_agent_handler = proc { |si, resp|
                resp.sub('REPLACE_ME', si)
              }
              script_insert = '<script>script_insert</script>'

              rack_body, content_legnth = JSAgent.insert_now(js_agent_handler,
                                                             script_insert,
                                                             ['first body part',
                                                              'second body part'],
                                                             'first body partsecond body part'.bytesize)
              expect(rack_body).to eq(['first body part', 'second body part'])
              expect(content_legnth).to eq(31)
            end
          end

          context 'that matches insert' do
            it 'should return modified body and new content length' do
              js_agent_handler = proc { |si, resp|
                resp.sub('REPLACE_ME', si)
              }
              script_insert = '<script>script_insert</script>'

              rack_body, content_legnth = JSAgent.insert_now(js_agent_handler,
                                                             script_insert,
                                                             ['first body part REPLACE_ME',
                                                              'second body part'],
                                                             'first body part REPLACE_MEsecond body part'.bytesize)

              expect(rack_body).to eq(['first body part <script>script_insert</script>',
                                       'second body part'])
              expect(content_legnth).to eq(62)
            end
          end
        end
      end

      describe '.handle_js_agent_insert' do
        context 'no <head> tag' do
          it 'should not modify response' do
            response = JSAgent.handle_js_agent_insert('SCRIPT', 'i am the response')

            expect(response).to eq('i am the response')
          end
        end

        context 'with a <head> tag' do
          it 'should append script after head tag' do
            response = JSAgent.handle_js_agent_insert('SCRIPT', 'i am the <head> response')

            expect(response).to eq('i am the <head>SCRIPT response')
          end
        end

        context 'with a <HEAD> tag' do
          it 'should append script after <HEAD> tag' do
            response = JSAgent.handle_js_agent_insert('SCRIPT', 'i am the <HEAD> response')

            expect(response).to eq('i am the <HEAD>SCRIPT response')
          end
        end

        context 'with a <header> tag' do
          it 'should not append script after <header> tag' do
            response = JSAgent.handle_js_agent_insert('SCRIPT', 'i am the <header> response')

            expect(response).to eq('i am the <header> response')
          end
        end

        context 'with a <HEAD\n> tag' do
          it 'should append script after <HEAD\n> tag' do
            response = JSAgent.handle_js_agent_insert('SCRIPT', "i am the <HEAD\n> response")

            expect(response).to eq("i am the <HEAD\n>SCRIPT response")
          end
        end

        context 'with invalid parameters' do
          context 'with nil response' do
            it 'should return the unmodified response' do
              logger = double('logger')
              expect(TCellAgent::Instrumentation).to receive(:get_safe_block_logger).and_return(logger)
              expect(logger).to receive(:error).with(
                /Error Handling JSAgent insert \(NoMethodError\): undefined method `sub'/
              )
              expect(logger).to receive(:exception) # exception stack trace

              response = JSAgent.handle_js_agent_insert('SCRIPT', nil)

              expect(response).to eq(nil)
            end
          end

          context 'with nil script' do
            it 'should return the unmodified response' do
              expect(TCellAgent).to_not receive(:logger)

              response = JSAgent.handle_js_agent_insert(nil, 'i am the <head> response')

              expect(response).to eq('i am the <head> response')
            end
          end
        end
      end

      describe '.get_handler_and_script_insert' do
        context 'with empty content type' do
          it 'should not set js_agent_handler or script_insert' do
            request = double('request')

            expect(TCellAgent).to_not receive(:policy)
            result = JSAgent.get_handler_and_script_insert(request, {})

            expect(result).to eq([nil, nil])
          end
        end

        context 'with non html response' do
          it 'should not set js_agent_handler or script_insert' do
            request = double('request')

            expect(TCellAgent).to_not receive(:policy)
            result = JSAgent.get_handler_and_script_insert(
              request,
              { 'Content-Type' => 'text/plain' }
            )

            expect(result).to eq([nil, nil])
          end
        end

        context 'with html response and js agent policy' do
          context 'with nil script_insert' do
            it 'should not set js_agent_handler or script_insert' do
              tcell_data = TCellAgent::Instrumentation::TCellData.new
              request = double('request',
                               :env => {
                                 TCellAgent::Instrumentation::TCELL_ID => tcell_data
                               })
              js_agent_policy = double('js_agent_policy')

              expect(TCellAgent).to receive(:policy).with(
                TCellAgent::PolicyTypes::JSAGENTINJECTION
              ).and_return(js_agent_policy)
              expect(js_agent_policy).to receive(:get_js_agent_script_tag).with(
                tcell_data
              ).and_return(nil)

              result = JSAgent.get_handler_and_script_insert(
                request,
                { 'Content-Type' => 'text/html' }
              )

              expect(result).to eq([nil, nil])
            end
          end

          context 'with a script_insert' do
            it 'should set js_agent_handler and script_insert' do
              tcell_data = TCellAgent::Instrumentation::TCellData.new
              request = double('request',
                               :env => {
                                 TCellAgent::Instrumentation::TCELL_ID => tcell_data
                               })
              js_agent_policy = double('js_agent_policy')

              expect(TCellAgent).to receive(:policy).with(
                TCellAgent::PolicyTypes::JSAGENTINJECTION
              ).and_return(js_agent_policy)
              expect(js_agent_policy).to receive(:get_js_agent_script_tag).with(
                tcell_data
              ).and_return('SCRIPT')

              js_agent_handler, script_insert = JSAgent.get_handler_and_script_insert(
                request,
                { 'Content-Type' => 'text/html' }
              )

              expect(js_agent_handler.class).to eq(Proc)
              expect(script_insert).to eq('SCRIPT')
            end
          end
        end
      end
    end
  end
end
