require 'spec_helper'

describe Logger do
  describe '#add' do
    context 'with a warn logger' do
      before(:each) do
        req_env = double('request_env')
        lfi_policy = double('lfi_policy')
        native_agent = double('native_agent')
        @local_files_policy = TCellAgent::Policies::LocalFileInclusion.new(
          native_agent, {}
        )
        allow(req_env).to receive(:[])
        allow(TCellAgent).to receive(:policy).with(TCellAgent::PolicyTypes::LFI).and_return(
          lfi_policy
        )
        allow(TCellAgent::Instrumentation::Rails::Middleware::ContextMiddleware::THREADS).to receive(
          :fetch
        ).with(anything, {}).and_return(req_env)
        allow(lfi_policy).to receive(:block_file_access?).and_return(false)
      end
      context 'writing a debug message' do
        it 'should skip the tcell logic' do
          expect(TCellAgent::Instrumentation).to_not receive(:safe_block_no_log)

          logger = Logger.new('/dev/null')

          logger.level = Logger::WARN
          logger.add(Logger::DEBUG, 'This will not be logged')
        end
      end

      context 'writing a warn message' do
        it 'should run the tcell logic' do
          expect(TCellAgent::Instrumentation).to receive(:safe_block_no_log).with(
            'Handling DLP log message filtering'
          )

          logger = Logger.new('/dev/null')

          logger.level = Logger::WARN
          logger.add(Logger::WARN, 'This will be logged')
        end
      end

      context 'writing an error message' do
        it 'should run the tcell logic' do
          expect(TCellAgent::Instrumentation).to receive(:safe_block_no_log).with(
            'Handling DLP log message filtering'
          )

          logger = Logger.new('/dev/null')

          logger.level = Logger::WARN
          logger.add(Logger::ERROR, 'This will be logged')
        end

        context 'with an empty message' do
          it 'should not run the context filter' do
            dlp_policy = double('dlp_policy', :enabled => true)
            expect(TCellAgent::Instrumentation).to receive(:safe_block_no_log).with(
              'Handling DLP log message filtering'
            ).and_call_original
            expect(TCellAgent).to receive(:policy).with(TCellAgent::PolicyTypes::DATALOSS).and_return(
              dlp_policy
            )
            expect(TCellAgent::Instrumentation::Rails::Middleware::ContextMiddleware::THREADS).to receive(
              :fetch
            ).with(anything, nil).and_return(double('request_env'))

            logger = Logger.new('/dev/null')

            logger.level = Logger::WARN
            logger.add(Logger::ERROR, nil)
          end
        end

        context 'with no dlp policy' do
          it 'should not run the context filter' do
            expect(TCellAgent::Instrumentation).to receive(:safe_block_no_log).with(
              'Handling DLP log message filtering'
            ).and_call_original
            expect(TCellAgent).to receive(:policy).with(TCellAgent::PolicyTypes::DATALOSS).and_return(
              nil
            )
            expect(TCellAgent::Instrumentation::Rails::Middleware::ContextMiddleware::THREADS).to_not receive(
              :fetch
            ).with(anything, nil)

            logger = Logger.new('/dev/null')

            logger.level = Logger::WARN
            logger.add(Logger::ERROR, 'My DLP Policy :(')
          end
        end

        context 'with no request env' do
          it 'should not run the context filter' do
            dlp_policy = double('dlp_policy', :enabled => true)
            expect(TCellAgent::Instrumentation).to receive(:safe_block_no_log).with(
              'Handling DLP log message filtering'
            ).and_call_original
            expect(TCellAgent).to receive(:policy).with(TCellAgent::PolicyTypes::DATALOSS).and_return(
              dlp_policy
            )
            expect(TCellAgent::Instrumentation::Rails::Middleware::ContextMiddleware::THREADS).to receive(
              :fetch
            ).with(anything, nil).and_return(nil)

            logger = Logger.new('/dev/null')

            logger.level = Logger::WARN
            logger.add(Logger::ERROR, 'My DLP Policy :(')
          end
        end

        context 'with a dlp policy, a message, and request env' do
          context 'with no tcell_context' do
            it 'should not run the context filter' do
              request_env = double('request_env')
              dlp_policy = double('dlp_policy', :enabled => true)

              expect(TCellAgent::Instrumentation).to receive(:safe_block_no_log).with(
                'Handling DLP log message filtering'
              ).and_call_original
              expect(TCellAgent).to receive(:policy).with(TCellAgent::PolicyTypes::DATALOSS).and_return(
                dlp_policy
              )
              expect(TCellAgent::Instrumentation::Rails::Middleware::ContextMiddleware::THREADS).to receive(
                :fetch
              ).with(anything, nil).and_return(request_env)
              expect(request_env).to receive(:[]).and_return(nil)

              logger = Logger.new('/dev/null')

              logger.level = Logger::WARN
              logger.add(Logger::ERROR, 'My DLP Policy :(')
            end
          end

          context 'with tcell_context' do
            it 'should run the context filter' do
              request_env = double('request_env')
              tcell_context = double('tcell_context')
              dlp_policy = double('dlp_policy', :enabled => true)

              expect(TCellAgent::Instrumentation).to receive(:safe_block_no_log).with(
                'Handling DLP log message filtering'
              ).and_call_original
              expect(TCellAgent).to receive(:policy).with(TCellAgent::PolicyTypes::DATALOSS).and_return(
                dlp_policy
              )
              expect(TCellAgent::Instrumentation::Rails::Middleware::ContextMiddleware::THREADS).to receive(
                :fetch
              ).with(anything, nil).and_return(request_env)
              expect(request_env).to receive(:[]).and_return(tcell_context)
              expect(tcell_context).to receive(:filter_log).with('My DLP Policy :(')

              logger = Logger.new('/dev/null')

              logger.level = Logger::WARN
              logger.add(Logger::ERROR, 'My DLP Policy :(')
            end
          end
        end
      end
    end
  end
end
