require 'spec_helper'

module TCellAgent
  module Instrumentation
    module Rails
      describe TCellBodyProxy do
        context '#close' do
          before(:each) do
            @meta_data = TCellAgent::MetaData.new(
              'get',
              'remote_address',
              'route_id',
              'session_id',
              'user_id',
              'transaction_id',
              'http://test.com/',
              '0.0.0.0'
            )
          end

          context 'zero content length' do
            it 'appfirewall injections should be checked' do
              tcell_body_proxy = TCellBodyProxy.new(
                Rack::BodyProxy.new(['body']) {},
                true,
                nil, nil, nil, nil
              )
              tcell_body_proxy.meta_data = @meta_data

              tcell_body_proxy.content_length = 0

              appfirewall_policy = double('appfirewall_policy')
              expect(TCellAgent::Instrumentation).to receive(:safe_block).with(
                'Running AppSensor deferred due to streaming'
              ).and_call_original
              expect(TCellAgent).to receive(:policy).with(
                TCellAgent::PolicyTypes::APPSENSOR
              ).and_return(appfirewall_policy)
              expect(appfirewall_policy).to receive(:check_appfirewall_injections).with(
                @meta_data
              )

              tcell_body_proxy.close
            end
          end

          context 'non zero content length' do
            it 'should check for appfirewall injections' do
              tcell_body_proxy = TCellBodyProxy.new(
                Rack::BodyProxy.new(['body']) {},
                true,
                nil, nil, nil, nil
              )
              tcell_body_proxy.meta_data = @meta_data

              tcell_body_proxy.content_length = 512

              appfirewall_policy = double('appfirewall_policy')
              expect(TCellAgent::Instrumentation).to receive(:safe_block).with(
                'Running AppSensor deferred due to streaming'
              ).and_call_original
              expect(TCellAgent).to receive(:policy).with(
                TCellAgent::PolicyTypes::APPSENSOR
              ).and_return(appfirewall_policy)
              expect(appfirewall_policy).to receive(:check_appfirewall_injections).with(
                @meta_data
              )

              tcell_body_proxy.close

              expect(@meta_data.response_content_bytes_len).to eq(512)
            end
          end
        end

        context '#each' do
          context 'with no block given' do
            it 'should return an enumerator' do
              tcell_body_proxy = TCellBodyProxy.new(
                Rack::BodyProxy.new(['body']) {},
                true,
                nil, nil, nil, nil
              )
              expect(tcell_body_proxy.each.class.name).to eq('Enumerator')
            end
          end

          context 'with a string response' do
            context 'with a chunked response' do
              it 'should only calculate content length' do
                tcell_body_proxy = TCellBodyProxy.new(
                  Rack::BodyProxy.new(["2d\r\nsome content\r\n"]) {},
                  false,
                  nil, nil, nil, nil
                )

                expect(TCellAgent::Instrumentation).to receive(:safe_block).with(
                  'Processing tcell body proxy body'
                ).and_call_original

                tcell_body_proxy.each { |b| }

                expect(tcell_body_proxy.content_length).to eq(45)
              end
            end

            context 'with a non chunked response' do
              context 'that should not be processed' do
                it 'should only calculate content length' do
                  tcell_body_proxy = TCellBodyProxy.new(
                    Rack::BodyProxy.new(['some content']) {},
                    false,
                    nil, nil, nil, nil
                  )

                  expect(TCellAgent::Instrumentation).to receive(:safe_block).with(
                    'Processing tcell body proxy body'
                  ).and_call_original

                  tcell_body_proxy.each { |b| }

                  expect(tcell_body_proxy.content_length).to eq(12)
                end
              end

              context 'that should be processed' do
                it 'should call js and dlp procs as well as calculate content length' do
                  js_agent_insertion_proc = double('js_agent_insertion_proc')
                  dlp_cleaner_proc = double('dlp_cleaner_proc')
                  tcell_body_proxy = TCellBodyProxy.new(
                    Rack::BodyProxy.new(['some content']) {},
                    true,
                    js_agent_insertion_proc, 'script_insert', dlp_cleaner_proc, nil
                  )

                  expect(TCellAgent::Instrumentation).to receive(:safe_block).with(
                    'Processing tcell body proxy body'
                  ).and_call_original
                  expect(js_agent_insertion_proc).to receive(:call).with(
                    'script_insert', 'some content'
                  ).and_return('some content')
                  expect(dlp_cleaner_proc).to receive(:call).with(nil, 'some content')

                  tcell_body_proxy.each { |b| }

                  expect(tcell_body_proxy.content_length).to eq(12)
                end
              end
            end
          end

          context 'with an ActionView::OutputBuffer' do
            context 'that should not be processed' do
              it 'should only calculate content length' do
                body_chunk = 'some content'
                js_agent_insertion_proc = double('js_agent_insertion_proc')
                dlp_cleaner_proc = double('dlp_cleaner_proc')
                tcell_body_proxy = TCellBodyProxy.new(
                  Rack::BodyProxy.new([body_chunk]) {},
                  false,
                  nil, nil, nil, nil
                )

                expect(TCellAgent::Instrumentation).to receive(:safe_block).with(
                  'Processing tcell body proxy body'
                ).and_call_original
                expect(body_chunk).to receive(:class).and_return(
                  double('body_class', :name => 'ActionView::OutputBuffer')
                )
                expect(body_chunk).to receive(:class).and_return(
                  double('body_class', :name => 'ActionView::OutputBuffer')
                )
                expect(js_agent_insertion_proc).to_not receive(:call)
                expect(dlp_cleaner_proc).to_not receive(:call)

                tcell_body_proxy.each { |b| }

                expect(tcell_body_proxy.content_length).to eq(12)
              end
            end

            context 'that should be processed' do
              it 'should call js and dlp procs as well as calculate content length' do
                body_chunk = 'some content'
                js_agent_insertion_proc = double('js_agent_insertion_proc')
                dlp_cleaner_proc = double('dlp_cleaner_proc')
                tcell_body_proxy = TCellBodyProxy.new(
                  Rack::BodyProxy.new([body_chunk]) {},
                  true,
                  js_agent_insertion_proc, 'script_insert', dlp_cleaner_proc, nil
                )

                expect(TCellAgent::Instrumentation).to receive(:safe_block).with(
                  'Processing tcell body proxy body'
                ).and_call_original
                expect(js_agent_insertion_proc).to receive(:call).with(
                  'script_insert', body_chunk
                ).and_return(body_chunk)
                expect(dlp_cleaner_proc).to receive(:call).with(nil, body_chunk)

                tcell_body_proxy.each { |b| }

                expect(tcell_body_proxy.content_length).to eq(12)
              end
            end
          end
        end
      end
    end
  end
end
