require 'spec_helper'

module TCellAgent
  module Instrumentation
    # Fake out grape gem
    module Grape
      class API
      end
    end

    class GrapeRoute < Grape::API
    end

    class RegularRoute
    end

    describe '.grape_route?' do
      context 'rails 3' do
        context 'with a regular route' do
          it 'should return false' do
            stub_const('::Rails::VERSION::MAJOR', 3)
            stub_const('::Rails::VERSION::MINOR', 0)

            route = double('route', :app => RegularRoute.new)

            is_grape_route = TCellAgent::Instrumentation.grape_route?(route)

            expect(is_grape_route).to eq(false)
          end
        end

        context 'with a grape route' do
          it 'should return false' do
            stub_const('::Rails::VERSION::MAJOR', 3)
            stub_const('::Rails::VERSION::MINOR', 0)

            route = double('route', :app => GrapeRoute)

            is_grape_route = TCellAgent::Instrumentation.grape_route?(route)

            expect(is_grape_route).to eq(false)
          end
        end
      end

      context 'rails 4.0' do
        context 'with a regular route' do
          it 'should return false' do
            stub_const('::Rails::VERSION::MAJOR', 4)
            stub_const('::Rails::VERSION::MINOR', 0)

            route = double('route', :app => RegularRoute.new)

            is_grape_route = TCellAgent::Instrumentation.grape_route?(route)

            expect(is_grape_route).to eq(false)
          end
        end

        context 'with a grape route' do
          it 'should return true' do
            stub_const('::Rails::VERSION::MAJOR', 4)
            stub_const('::Rails::VERSION::MINOR', 0)

            route = double('route', :app => GrapeRoute)

            is_grape_route = TCellAgent::Instrumentation.grape_route?(route)

            expect(is_grape_route).to eq(true)
          end
        end
      end

      context 'rails 4.1' do
        context 'with a regular route' do
          it 'should return false' do
            stub_const('::Rails::VERSION::MAJOR', 4)
            stub_const('::Rails::VERSION::MINOR', 1)

            route = double('route', :app => RegularRoute.new)

            is_grape_route = TCellAgent::Instrumentation.grape_route?(route)

            expect(is_grape_route).to eq(false)
          end
        end

        context 'with a grape route' do
          it 'should return true' do
            stub_const('::Rails::VERSION::MAJOR', 4)
            stub_const('::Rails::VERSION::MINOR', 1)

            route = double('route', :app => GrapeRoute)

            is_grape_route = TCellAgent::Instrumentation.grape_route?(route)

            expect(is_grape_route).to eq(true)
          end
        end
      end

      context 'rails 4.2' do
        context 'with a regular route' do
          it 'should return false' do
            stub_const('::Rails::VERSION::MAJOR', 4)
            stub_const('::Rails::VERSION::MINOR', 2)

            route = double('route', :app => double('app', :app => RegularRoute.new))

            is_grape_route = TCellAgent::Instrumentation.grape_route?(route)

            expect(is_grape_route).to eq(false)
          end
        end

        context 'with a grape route' do
          it 'should return true' do
            stub_const('::Rails::VERSION::MAJOR', 4)
            stub_const('::Rails::VERSION::MINOR', 2)

            route = double('route', :app => double('app', :app => GrapeRoute))

            is_grape_route = TCellAgent::Instrumentation.grape_route?(route)

            expect(is_grape_route).to eq(true)
          end
        end
      end

      context 'rails 5.0' do
        context 'with a regular route' do
          it 'should return false' do
            stub_const('::Rails::VERSION::MAJOR', 5)
            stub_const('::Rails::VERSION::MINOR', 0)

            route = double('route', :app => double('app', :app => RegularRoute.new))

            is_grape_route = TCellAgent::Instrumentation.grape_route?(route)

            expect(is_grape_route).to eq(false)
          end
        end

        context 'with a grape route' do
          it 'should return true' do
            stub_const('::Rails::VERSION::MAJOR', 5)
            stub_const('::Rails::VERSION::MINOR', 0)

            route = double('route', :app => double('app', :app => GrapeRoute))

            is_grape_route = TCellAgent::Instrumentation.grape_route?(route)

            expect(is_grape_route).to eq(true)
          end
        end
      end
    end

    describe '.grape_route_info' do
      context 'with grape version 0.10.1' do
        it 'should use route_method and route_path' do
          route = double('route')
          grape_gem = double('grape_gem', :version => '0.10.1')

          expect(Gem).to receive(:loaded_specs).and_return({
                                                             'grape' => grape_gem
                                                           })
          expect(route).to receive(:route_method).and_return('POST')
          expect(route).to receive(:route_path).and_return('/grape_api/endpoint')

          route_info = TCellAgent::Instrumentation.grape_route_info(route)

          expect(route_info[:method]).to eq('POST')
          expect(route_info[:path]).to eq('/grape_api/endpoint')
        end
      end

      context 'with grape version 0.16.0' do
        it 'should use request_method and path' do
          route = double('route')
          grape_gem = double('grape_gem', :version => '0.16.0')

          expect(Gem).to receive(:loaded_specs).and_return({
                                                             'grape' => grape_gem
                                                           })
          expect(route).to receive(:request_method).and_return('POST')
          expect(route).to receive(:path).and_return('/grape_api/endpoint')

          route_info = TCellAgent::Instrumentation.grape_route_info(route)

          expect(route_info[:method]).to eq('POST')
          expect(route_info[:path]).to eq('/grape_api/endpoint')
        end
      end
    end

    describe '.instrument_grape_api' do
      context 'with nil routes' do
        it 'should not send any events' do
          expect(TCellAgent::SensorEvents::Util).to_not receive(:calculate_route_id)
          TCellAgent::Instrumentation.instrument_grape_api('/api', nil)
        end
      end

      context 'with empty routes' do
        it 'should not send any events' do
          expect(TCellAgent::SensorEvents::Util).to_not receive(:calculate_route_id)
          TCellAgent::Instrumentation.instrument_grape_api('/api', [])
        end
      end

      context 'with grape version < 1.16' do
        before(:each) do
          grape_gem = double('grape_gem', :version => '0.10.1')

          expect(Gem).to receive(:loaded_specs).and_return({
                                                             'grape' => grape_gem
                                                           }).at_least(1)
        end

        context 'with one route' do
          it 'should send one event' do
            route = double('grape_route', :route_method => 'GET', :route_path => '/users')

            expect(TCellAgent::SensorEvents::AppRoutesSensorEvent).to receive(:new).with(
              '/api/users', 'GET', '409553642', nil, nil
            ).and_call_original

            TCellAgent::Instrumentation.instrument_grape_api('/api', [route])
          end
        end

        context 'with two routes' do
          it 'should send two events' do
            route = double('grape_route', :route_method => 'GET', :route_path => '/users')
            route_dos = double('grape_route', :route_method => 'POST', :route_path => '/users')

            expect(TCellAgent::SensorEvents::AppRoutesSensorEvent).to receive(:new).with(
              '/api/users', 'GET', '409553642', nil, nil
            ).and_call_original
            expect(TCellAgent::SensorEvents::AppRoutesSensorEvent).to receive(:new).with(
              '/api/users', 'POST', '-2105512096', nil, nil
            ).and_call_original

            TCellAgent::Instrumentation.instrument_grape_api('/api', [route, route_dos])
          end
        end
      end

      context 'with grape version >= 1.16' do
        before(:each) do
          grape_gem = double('grape_gem', :version => '0.16.0')

          expect(Gem).to receive(:loaded_specs).and_return({
                                                             'grape' => grape_gem
                                                           }).at_least(1)
        end

        context 'with one route' do
          it 'should send one event' do
            route = double('grape_route', :request_method => 'GET', :path => '/users')

            expect(TCellAgent::SensorEvents::AppRoutesSensorEvent).to receive(:new).with(
              '/api/users', 'GET', '409553642', nil, nil
            ).and_call_original

            TCellAgent::Instrumentation.instrument_grape_api('/api', [route])
          end
        end

        context 'with two routes' do
          it 'should send two events' do
            route = double('grape_route', :request_method => 'GET', :path => '/users')
            route_dos = double('grape_route', :request_method => 'POST', :path => '/users')

            expect(TCellAgent::SensorEvents::AppRoutesSensorEvent).to receive(:new).with(
              '/api/users', 'GET', '409553642', nil, nil
            ).and_call_original
            expect(TCellAgent::SensorEvents::AppRoutesSensorEvent).to receive(:new).with(
              '/api/users', 'POST', '-2105512096', nil, nil
            ).and_call_original

            TCellAgent::Instrumentation.instrument_grape_api('/api', [route, route_dos])
          end
        end
      end
    end
  end
end
