require 'spec_helper'

module TCellAgent
  module Instrumentation
    # Fake out grape gem
    module Grape
      class API
      end
    end

    class GrapeRoute < Grape::API
      def self.routes
        ['/api/grape/route']
      end
    end

    describe '.instrument_route' do
      context 'with a nil route' do
        it "shouldn't be reported" do
          configuration = double('configuration', :should_instrument? => true)

          expect(TCellAgent).to receive(:configuration).and_return(configuration)
          expect(TCellAgent::Instrumentation).to_not receive(:instrument_grape_api)
          expect(TCellAgent::SensorEvents::Util).to_not receive(:calculate_route_id)

          Rails.instrument_route(nil)
        end
      end

      context 'with rails 5.0' do
        context 'with a regular route' do
          it 'should report it' do
            stub_const('::Rails::VERSION::MAJOR', 5)
            stub_const('::Rails::VERSION::MINOR', 0)

            configuration = double('configuration', :should_instrument? => true)
            route = double(
              'route',
              :path => double('path', :spec => '/regular/route'),
              :defaults => {},
              :app => double('app', :app => 'some_app')
            )

            expect(TCellAgent).to receive(:configuration).and_return(configuration)
            expect(route).to receive(:verb).and_return('GET|POST').twice
            expect(TCellAgent::SensorEvents::AppRoutesSensorEvent).to receive(:new).with(
              '/regular/route', 'GET', '-1187510355', nil, '{}'
            ).and_call_original
            expect(TCellAgent::SensorEvents::AppRoutesSensorEvent).to receive(:new).with(
              '/regular/route', 'POST', '1100742947', nil, '{}'
            ).and_call_original

            Rails.instrument_route(route)
          end
        end

        context 'with a grape route' do
          it 'should instrument the grape endpoint' do
            stub_const('::Rails::VERSION::MAJOR', 5)
            stub_const('::Rails::VERSION::MINOR', 0)

            configuration = double('configuration', :should_instrument? => true)
            route = double(
              'route',
              :path => double('path', :spec => '/grape'),
              :defaults => {},
              :app => double('app', :app => GrapeRoute)
            )

            expect(TCellAgent).to receive(:configuration).and_return(configuration)
            expect(route).to receive(:verb).and_return('GET|POST')
            expect(TCellAgent::SensorEvents::Util).to_not receive(:calculate_route_id)
            expect(TCellAgent::Instrumentation).to receive(:instrument_grape_api).with(
              '/grape', ['/api/grape/route']
            )

            Rails.instrument_route(route)
          end
        end
      end

      context 'with rails 4.2' do
        context 'with a regular route' do
          it 'should report it' do
            stub_const('::Rails::VERSION::MAJOR', 4)
            stub_const('::Rails::VERSION::MINOR', 2)

            configuration = double('configuration', :should_instrument? => true)
            route = double(
              'route',
              :path => double('path', :spec => '/regular/route'),
              :defaults => {},
              :constraints => { :request_method => 'DELETE' },
              :app => double('app', :app => 'some_app')
            )

            expect(TCellAgent).to receive(:configuration).and_return(configuration)
            expect(TCellAgent::Instrumentation).to_not receive(:instrument_grape_api)
            expect(route).to receive(:verb).and_return(/DELETE/).at_least(9)
            expect(TCellAgent::SensorEvents::AppRoutesSensorEvent).to receive(:new).with(
              '/regular/route', 'DELETE', '-990446408', nil, '{}'
            ).and_call_original

            Rails.instrument_route(route)
          end
        end

        context 'with a grape route' do
          it 'should instrument the grape endpoint' do
            stub_const('::Rails::VERSION::MAJOR', 4)
            stub_const('::Rails::VERSION::MINOR', 2)

            configuration = double('configuration', :should_instrument? => true)
            route = double(
              'route',
              :path => double('path', :spec => '/grape'),
              :defaults => {},
              :constraints => { :request_method => 'PUT' },
              :app => double('app', :app => GrapeRoute)
            )

            expect(TCellAgent).to receive(:configuration).and_return(configuration)
            expect(TCellAgent::Instrumentation).to receive(:instrument_grape_api).with(
              '/grape', ['/api/grape/route']
            )
            expect(TCellAgent::SensorEvents::Util).to_not receive(:calculate_route_id)

            Rails.instrument_route(route)
          end
        end
      end

      context 'with rails 4.1' do
        context 'with a regular route' do
          it 'should report it' do
            stub_const('::Rails::VERSION::MAJOR', 4)
            stub_const('::Rails::VERSION::MINOR', 1)

            configuration = double('configuration', :should_instrument? => true)
            route = double(
              'route',
              :path => double('path', :spec => '/regular/route'),
              :defaults => {},
              :constraints => { :request_method => 'PUT' },
              :app => 'some_app'
            )

            expect(TCellAgent).to receive(:configuration).and_return(configuration)
            expect(TCellAgent::Instrumentation).to_not receive(:instrument_grape_api)
            expect(route).to receive(:verb).and_return(/PUT/).at_least(9)
            expect(TCellAgent::SensorEvents::AppRoutesSensorEvent).to receive(:new).with(
              '/regular/route', 'PUT', '-1393108268', nil, '{}'
            ).and_call_original

            Rails.instrument_route(route)
          end
        end

        context 'with a grape route' do
          it 'should instrument the grape endpoint' do
            stub_const('::Rails::VERSION::MAJOR', 4)
            stub_const('::Rails::VERSION::MINOR', 1)
            configuration = double('configuration', :should_instrument? => true)
            route = double(
              'route',
              :path => double('path', :spec => '/grape'),
              :defaults => {},
              :constraints => { :request_method => 'PUT' },
              :app => GrapeRoute
            )

            expect(TCellAgent).to receive(:configuration).and_return(configuration)
            expect(TCellAgent::Instrumentation).to receive(:instrument_grape_api).with(
              '/grape', ['/api/grape/route']
            )
            expect(TCellAgent::SensorEvents::Util).to_not receive(:calculate_route_id)

            Rails.instrument_route(route)
          end
        end
      end
    end
  end
end
